# -*- coding:utf-8 -*-

import pytest
from unittest import TestCase
from unittest.mock import Mock, patch

from vault.tests.fakes import fake_request
from identity.tests.fakes import FakeResource, FakeToken
from identity.views import (ListUserView, CreateUserView, UpdateUserView,
                            DeleteUserView, UpdateProjectUserPasswordView)
from django.utils.translation import gettext as _


class ListUserTest(TestCase):

    def setUp(self):
        self.view = ListUserView.as_view()
        self.request = fake_request(method='GET')

        # does not connect to the keystone client
        patch('keystoneclient.v3.client.Client').start()

    def tearDown(self):
        patch.stopall()

    def test_list_users_needs_authentication(self):
        req = fake_request(method='GET', user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_show_user_list(self):
        patch('identity.keystone.Keystone.user_list',
              Mock(return_value=[FakeResource(1)])).start()

        self.request.user.token = FakeToken
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        response = self.view(self.request)

        response.render()

        self.assertIn(b'<td>FakeResource1</td>', response.content)

    @patch('identity.keystone.Keystone.user_list')
    def test_list_user_view_exception(self, mock_user_list):
        mock_user_list.side_effect = Exception()
        self.request.user.token = FakeToken

        response = self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(msgs[0].message, _('Unable to list users'))


class CreateUserTest(TestCase):

    def setUp(self):
        self.view = CreateUserView.as_view()

        self.request = fake_request()
        self.request.META.update({
            'SERVER_NAME': 'globo.com',
            'SERVER_PORT': '80'
        })
        self.request.user.token = FakeToken

        patch('actionlogger.actionlogger.ActionLogger.log',
              Mock(return_value=None)).start()

        patch('identity.keystone.Keystone._create_keystone_connection').start()

        patch('identity.keystone.Keystone.project_list',
              Mock(return_value=[FakeResource(1, 'project1')])).start()

        patch('identity.keystone.Keystone.role_list',
              Mock(return_value=[FakeResource(1, 'role1')])).start()

        # does not connect to the keystone client
        patch('keystoneclient.v3.client.Client').start()

    def tearDown(self):
        patch.stopall()

    def test_create_user_needs_authentication(self):
        req = fake_request(user=False)
        self.request.user.token = None

        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_ensure_project_list_was_filled_up(self):
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        response = self.view(self.request)
        response.render()

        self.assertIn(b'<option value="1">project1</option>', response.content)

    def test_ensure_role_list_was_filled_up(self):
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        response = self.view(self.request)
        response.render()

        self.assertIn(b'<option value="1">role1</option>', response.content)

    def test_enabled_field_is_a_select_tag(self):
        from django.forms.widgets import Select
        enabled_field = CreateUserView.form_class.base_fields['enabled']

        self.assertIsInstance(enabled_field.widget, Select)

    def test_ensure_enabled_field_initial_value_is_true(self):
        enabled_field = CreateUserView.form_class.base_fields['enabled']

        self.assertTrue(enabled_field)

    def test_validating_a_blank_field(self):
        user = FakeResource(1, 'user1')
        user.to_dict = lambda: {'name': user.name}
        user.default_project_id = 1
        user.project_id = 1

        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=user)).start()

        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()
        post.update({'name': '', 'enabled': True, 'id': 1, 'project': 1,
            'role': 1, 'password': 'aaa', 'password_confirm': 'aaa',
            'email': 'a@a.net'})
        self.request.POST = post

        response = self.view(self.request)
        response.render()

        self.assertIn(_('This field is required'), response.content.decode('UTF-8'))

    def test_validating_email_field(self):
        user = FakeResource(1, 'user1')
        user.to_dict = lambda: {'name': user.name}
        user.default_project_id = 1
        user.project_id = 1

        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=user)).start()

        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()
        post.update({'name': 'aaa', 'enabled': True, 'id': 1, 'project': 1,
            'role': 1, 'password': 'aaa', 'password_confirm': 'aaa',
            'email': 'a.a.net'})
        self.request.POST = post

        response = self.view(self.request)
        response.render()

        self.assertIn(_('Enter a valid email address'), response.content.decode('UTF-8'))

    @patch('identity.keystone.Keystone.user_create')
    def test_user_create_method_was_called(self, mock):

        self.request.method = 'POST'
        post = self.request.POST.copy()
        post.update({'name': 'aaa', 'enabled': True, 'id': 1, 'project': 1,
                     'role': 1, 'password': 'aaa', 'password_confirm': 'aaa',
                     'email': 'a@a.net'})
        self.request.POST = post

        self.view(self.request)

        mock.assert_called_with(name='aaa', enabled=True,
            project_id=1, role_id=1, password='aaa', email='a@a.net', domain=None)

    @patch('identity.keystone.Keystone.user_create')
    def test_create_user_view_exception(self, mock_user_create):
        mock_user_create.side_effect = Exception()

        self.request.method = 'POST'
        post = self.request.POST.copy()
        post.update({'name': 'aaa', 'enabled': True, 'id': 1, 'project': 1,
                     'role': 1, 'password': 'aaa', 'password_confirm': 'aaa',
                     'email': 'a@a.net'})
        self.request.POST = post

        self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(msgs[0].message, _('Error when creating user'))


class UpdateTeamsUsersViewTest(TestCase):

    def setUp(self):
        self.view = UpdateUserView.as_view()

        self.request = fake_request()
        self.request.META.update({
            'SERVER_NAME': 'globo.com',
            'SERVER_PORT': '80'
        })
        self.request.user.is_superuser = False
        self.request.user.token = FakeToken

        patch('actionlogger.actionlogger.ActionLogger.log',
              Mock(return_value=None)).start()

    def tearDown(self):
        patch.stopall()

    def test_update_teams_users_needs_authentication(self):
        req = fake_request(user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)


class UpdateUserTest(TestCase):

    def setUp(self):
        self.view = UpdateUserView.as_view()

        self.request = fake_request()
        self.request.META.update({
            'SERVER_NAME': 'globo.com',
            'SERVER_PORT': '80'
        })
        self.request.user.token = FakeToken

        patch('actionlogger.actionlogger.ActionLogger.log',
              Mock(return_value=None)).start()

        # does not connect to the keystone client
        patch('keystoneclient.v3.client.Client').start()

    def tearDown(self):
        patch.stopall()

    def test_update_user_needs_authentication(self):
        req = fake_request(user=False)
        self.request.user.token = None

        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    @patch('identity.keystone.Keystone.user_update')
    def test_user_update_method_was_called(self, mock_user_update):

        patch('identity.keystone.Keystone.project_list',
            Mock(return_value=[FakeResource(1, 'project1')])).start()

        patch('identity.keystone.Keystone.project_get',
            Mock(return_value=1)).start()

        user = FakeResource(1, 'user1')
        user.to_dict = lambda: {'name': user.name}
        user.default_project_id = 1

        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=user)).start()

        project = FakeResource(1, 'project1')
        project.to_dict = lambda: {'name': project.name}
        patch('identity.keystone.Keystone.project_get',
              Mock(return_value=project)).start()

        self.request.method = 'POST'

        post = self.request.POST.copy()
        post.update({'id': 1, 'name': 'aaa', 'project': 1})
        self.request.POST = post

        self.view(self.request)

        mock_user_update.assert_called_with(user, name='aaa', project=project.id,
                                            domain=None, enabled=True,
                                            password=None, email=None)

    @patch('identity.keystone.Keystone.user_update')
    def test_update_user_view_exception(self, mock_user_update):
        mock_user_update.side_effect = Exception()

        patch('identity.keystone.Keystone.project_list',
            Mock(return_value=[FakeResource(1, 'project1')])).start()

        patch('identity.keystone.Keystone.project_get',
            Mock(return_value=1)).start()

        user = FakeResource(1, 'user1')
        user.to_dict = lambda: {'name': user.name}
        user.default_project_id = 1

        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=user)).start()

        self.request.method = 'POST'
        post = self.request.POST.copy()
        post.update({'id': 1, 'name': 'aaa', 'project': 1})
        self.request.POST = post

        self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(msgs[0].message, _('Error when updating user'))

    @patch('identity.keystone.Keystone.user_update')
    def test_update_user_change_password_exception(self, mock_user_update):
        mock_user_update.side_effect = Exception()

        patch('identity.keystone.Keystone.project_list',
            Mock(return_value=[FakeResource(1, 'project1')])).start()

        patch('identity.keystone.Keystone.project_get',
            Mock(return_value=1)).start()

        user = FakeResource(1, 'user1')
        user.to_dict = lambda: {'name': user.name}
        user.default_project_id = 1

        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=user)).start()

        self.request.method = 'POST'
        self.request.META.update({
            'HTTP_HOST': 'localhost'
        })
        post = self.request.POST.copy()
        post.update({
            'id': 1,
            'name': 'aaa',
            'project': 1,
            'password': 'globo123',
            'password_confirm': 'globo',
        })
        self.request.POST = post

        response = self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertIn(_('Passwords did not match'), response.rendered_content)
        self.assertEqual(len(msgs), 0)


class DeleteUserTest(TestCase):

    def setUp(self):
        self.view = DeleteUserView.as_view()

        self.request = fake_request()
        self.request.META.update({
            'SERVER_NAME': 'globo.com',
            'SERVER_PORT': '80'
        })
        self.request.user.token = FakeToken

        patch('actionlogger.actionlogger.ActionLogger.log',
              Mock(return_value=None)).start()

        patch('identity.keystone.Keystone._create_keystone_connection').start()

    def tearDown(self):
        patch.stopall()

    def test_delete_user_needs_authentication(self):
        req = fake_request(user=False)
        self.request.user.token = None

        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    @patch('identity.keystone.Keystone.user_delete')
    def test_user_delete_method_was_called(self, mock_user_delete):
        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=FakeResource(1, 'user1'))).start()

        self.view(self.request, user_id=1)

        mock_user_delete.assert_called_with(1)

    @patch('identity.keystone.Keystone.user_delete')
    def test_user_delete_sucessfull_message(self, mock_user_delete):
        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=FakeResource(1, 'user1'))).start()

        self.view(self.request, user_id=1)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(msgs[0].message, _('Successfully deleted user'))

    @patch('identity.keystone.Keystone.user_delete')
    def test_user_delete_view_exception(self, mock_user_delete):
        mock_user_delete.side_effect = Exception()

        patch('identity.keystone.Keystone.user_get',
              Mock(return_value=FakeResource(1, 'user1'))).start()

        self.view(self.request, user_id=1)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(msgs[0].message, _('Error when deleting user'))


class UpdateProjectUserPasswordTest(TestCase):

    def setUp(self):
        self.view = UpdateProjectUserPasswordView.as_view()

        self.request = fake_request(method='POST')
        post = self.request.POST.copy()
        post.update({"project": "AProjectID"})
        self.request.POST = post

        self.mock_keystone_find_user = patch('identity.keystone.Keystone.find_user_with_u_prefix').start()
        self.mock_keystone_find_user.return_value = self.request.user

        self.mock_users_list = patch('identity.keystone.Keystone.user_list').start()
        self.mock_users_list.return_value = [fake_request(method='GET').user]

        patch('identity.keystone.Keystone._create_keystone_connection').start()

    def tearDown(self):
        patch.stopall()

    def test_reset_password_needs_authentication(self):
        req = fake_request(user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    @patch('identity.keystone.Keystone.find_user_with_u_prefix')
    def test_find_user_with_u_prefix_was_called(self, mock_find_user):
        self.view(self.request)

        mock_find_user.assert_called_with("AProjectID", 'u')

    @patch('identity.keystone.Keystone.user_update')
    def test_user_update_was_called(self, mock_user_update):
        password = 'B52j7#ZDYuyS'

        patch('identity.keystone.Keystone.create_password',
              Mock(return_value=password)).start()

        self.view(self.request)

        mock_user_update.assert_called_with(self.request.user, password=password)

    @patch('identity.keystone.Keystone.user_update')
    def test_return_reset_password_with_new_password(self, mock_user_update):
        password = 'B52j7#ZDYuyS'

        patch('identity.keystone.Keystone.create_password',
              Mock(return_value=password)).start()

        response = self.view(self.request)

        mock_user_update.assert_called_with(self.request.user, password=password)

        self.assertIn(password, response.content.decode())
        self.assertEqual(response.status_code, 200)

    @patch('identity.keystone.Keystone.find_user_with_u_prefix')
    def test_find_user_with_u_prefix_with_exception(self, mock_find_u_user):
        mock_find_u_user.side_effect = Exception()

        response = self.view(self.request)

        self.assertEqual(response.status_code, 500)
        self.assertIn(_('Error updating password'), response.content.decode())



from django.conf.urls import re_path
from identity import views


urlpatterns = [
    # Admin Users Keystone
    re_path(r"^users/?$", views.ListUserView.as_view(), name="admin_list_users"),
    re_path(r"^user/add/?$", views.CreateUserView.as_view(), name="admin_add_user"),
    re_path(r"^user/(?P<user_id>[\w\-]+)/?$", views.UpdateUserView.as_view(), name="edit_user"),
    re_path(r"^user/delete/(?P<user_id>[\w\-]+)/?$", views.DeleteUserView.as_view(), name="delete_user"),

    # Ajax
    re_path(r"^project-add-user/?$", views.AddUserRoleView.as_view(), name="project_add_user"),
    re_path(r"^project-delete-user/?$", views.DeleteUserRoleView.as_view(), name="project_delete_user"),
    re_path(r"^project-list-users/?$", views.ListUserRoleView.as_view(), name="project_list_users"),

    # Project
    re_path(r"^projects/", views.ListProjectView.as_view(), name="projects"),
    re_path(r"^project/created/?$", views.CreateProjectSuccessView.as_view(), name="create_project_success"),
    re_path(r"^project/(?P<project_id>[\w\-]+)/?$", views.UpdateProjectView.as_view(), name="edit_project"),
    re_path(r"^project/delete/(?P<project_id>[\w\-]+)/?$", views.DeleteProjectView.as_view(), name="delete_project"),

    # Update User Password
    re_path(r"^project/user/updatepass/?$", views.UpdateProjectUserPasswordView.as_view(), name="update_pass"),

    # API
    re_path(r"^api/info$", views.JsonInfoView.as_view(), name="info_json"),
]



# Generated by Django 3.0.3 on 2020-05-25 12:02

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('project', models.CharField(max_length=255)),
                ('user', models.CharField(max_length=255)),
                ('password', models.CharField(max_length=255)),
                ('created_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'project',
                'unique_together': {('project',)},
            },
        ),
    ]



# -*- coding: utf-8 -*-

from .settings import *


DATABASES = {
    'default': {
        'ENGINE': os.getenv('DATABASES_DEFAULT_ENGINE', 'django.db.backends.sqlite3'),
        'NAME': os.getenv('DATABASES_DEFAULT_NAME', 'vault_test.db'),
    }
}



# -*- coding:utf-8 -*-

import json

from unittest import TestCase
from unittest.mock import patch

from identity.views import (
    ListUserRoleView, AddUserRoleView, DeleteUserRoleView)
from identity.tests.fakes import FakeToken, FakeResource
from vault.tests.fakes import fake_request, UserFactory


class BaseAjaxTestCase(TestCase):
    view_class = None

    def setUp(self):
        self.view = self.view_class.as_view()
        self.request = fake_request(method='POST')

        self.request.user.token = FakeToken
        self.request.user.is_superuser = True

        patch('identity.keystone.Keystone._create_keystone_connection').start()
        patch('identity.views.log').start()

    def tearDown(self):
        patch.stopall()


class TestListUserRole(BaseAjaxTestCase):
    view_class = ListUserRoleView

    def setUp(self):
        super(TestListUserRole, self).setUp()
        self.mock_user_list = patch(
            'identity.keystone.Keystone.user_list').start()

    def test_list_user_role_needs_authentication(self):
        req = fake_request(method='POST', user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_list_user_role_need_to_be_superuser(self):
        self.request.user.is_superuser = False

        response = self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(msgs[0].message, 'Unauthorized')

    def test_list_user_role_response_content_is_json(self):
        response = self.view(self.request)
        self.assertEqual(response._headers.get(
            'content-type')[1], 'application/json')

    @patch('identity.views.ListUserRoleView.get_user_roles')
    def test_list_user_role_return_sorted_users(self, mock_get_user_roles):
        user1 = FakeResource(1)
        user1.name = 'ZZZ'

        user2 = FakeResource(2)
        user2.name = 'BBBB'

        user3 = FakeResource(3)
        user3.name = 'LLLL'

        self.mock_user_list.return_value = [user1, user2, user3]
        mock_get_user_roles.return_value = []

        post = self.request.POST.copy()
        post.update({'project': 1})
        self.request.POST = post

        response = self.view(self.request)

        computed = json.loads(response.content)
        computed_names = [x.get('username') for x in computed.get('users')]
        expected = sorted([user1.name, user2.name, user3.name])

        self.assertEqual(computed_names, expected)

    def test_user_list_was_called(self):
        post = self.request.POST.copy()
        post.update({'project': 1})
        self.request.POST = post

        self.view(self.request)

        self.mock_user_list.assert_called_with(project_id=1)

    @patch('identity.views.ListUserRoleView.get_user_roles')
    def test_get_user_roles_was_called(self, mock_get_user_roles):
        user = FakeResource(1)
        user.username = 'User1'

        self.mock_user_list.return_value = [user]
        mock_get_user_roles.return_value = []

        post = self.request.POST.copy()
        post.update({'project': 1})
        self.request.POST = post

        self.view(self.request)

        mock_get_user_roles.assert_called_with(user, 1)

    @patch('identity.views.ListUserRoleView.get_user_roles')
    def test_get_user_roles_exception(self, mock_get_user_roles):
        mock_get_user_roles.side_effect = Exception()
        mock_get_user_roles.return_value = []

        user = FakeResource(1)
        user.username = 'User1'

        self.mock_user_list.return_value = [user]

        post = self.request.POST.copy()
        post.update({'project': 1})
        self.request.POST = post

        response = self.view(self.request)

        self.assertEqual(response.status_code, 500)


class TestAddUserRole(BaseAjaxTestCase):
    view_class = AddUserRoleView

    def test_add_user_role_needs_authentication(self):
        req = fake_request(method='POST', user=False)
        self.request.user.token = None
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_add_user_role_need_to_be_superuser(self):
        self.request.user.is_superuser = False

        response = self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(msgs[0].message, 'Unauthorized')

    @patch('identity.keystone.Keystone.add_user_role')
    def test_add_user_role_response_content_is_json(self, mock_add_user_role):
        response = self.view(self.request)

        self.assertEqual(response._headers.get(
            'content-type')[1], 'application/json')

    @patch('identity.keystone.Keystone.add_user_role')
    def test_add_user_role_was_called(self, mock_add_user_role):
        post = self.request.POST.copy()
        post.update({'project': 1, 'role': 1, 'user': 1})
        self.request.POST = post

        self.view(self.request)

        mock_add_user_role.assert_called_with(project=1, role=1, user=1)

    @patch('identity.keystone.Keystone.add_user_role')
    def test_add_user_role_exception(self, mock_add_user_role):
        mock_add_user_role.side_effect = Exception()

        response = self.view(self.request)

        self.assertEqual(response.status_code, 500)


class TestDeleteUserRole(BaseAjaxTestCase):
    view_class = DeleteUserRoleView

    def test_delete_user_role_needs_authentication(self):
        req = fake_request(method='POST', user=False)
        self.request.user.token = None

        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_delete_user_role_need_to_be_superuser(self):
        self.request.user.is_superuser = False

        response = self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(msgs[0].message, 'Unauthorized')

    @patch('identity.keystone.Keystone.remove_user_role')
    def test_delete_user_role_response_content_is_json(self, mock_remove_user_role):
        response = self.view(self.request)

        self.assertEqual(response._headers.get(
            'content-type')[1], 'application/json')

    @patch('identity.keystone.Keystone.remove_user_role')
    def test_remove_user_role_was_called(self, mock_remove_user_role):
        post = self.request.POST.copy()
        post.update({'project': 1, 'role': 1, 'user': 1})
        self.request.POST = post

        self.view(self.request)

        mock_remove_user_role.assert_called_with(project=1, role=1, user=1)

    @patch('identity.keystone.Keystone.remove_user_role')
    def test_remove_user_role_exception(self, mock_remove_user_role):
        mock_remove_user_role.side_effect = Exception()

        response = self.view(self.request)

        self.assertEqual(response.status_code, 500)



# -*- coding: utf-8 -*-

from django.conf.urls import re_path
from storage import views


urlpatterns = [
    # Trash: enable/disable Undelete
    re_path(r'^trash-container-config/(?P<container>.+?)$', views.config_trash_container, name="config_trash_container"),
    re_path(r'^trash-container-status/(?P<container>.+?)$', views.container_trash_status, name="container_trash_status"),

    # Backup
    re_path(r'^backup-container-config/(?P<container>.+?)$', views.config_backup_container, name="config_backup_container"),
    re_path(r'^backup-container-status/(?P<container>.+?)$', views.container_backup_status, name="container_backup_status"),

    # Restore
    re_path(r'^backup-restore/', views.backup_restore, name="backup_restore"),

    # ACL: make container public/private
    re_path(r'^acl-container-update/(?P<container>.+?)$', views.container_acl_update, name="container_acl_update"),
    re_path(r'^acl-container-status/(?P<container>.+?)$', views.container_acl_status, name="container_acl_status"),

    # Cache
    re_path(r'^cache/?$', views.storage_cache, name="storage_cache"),
    re_path(r'^cache/remove/?$', views.remove_from_cache, name="remove_from_cache"),
    re_path(r'^cache/add/?$', views.add_to_cache, name="add_to_cache"),

    # Trash
    re_path(r'^trash/restore/', views.restore_object, name="restore_object"),
    re_path(r'^trash/remove/', views.remove_from_trash, name="remove_from_trash"),
    re_path(r'^trash/(?P<container>.+?)/(?P<prefix>(.+)+)?$', views.get_deleted_objects, name="deleted_objects"),

    # Storage urls
    re_path(r'^account/?$', views.accountview, name="accountview"),
    re_path(r'^objects/(?P<container>.+?)/(?P<prefix>(.+)+)?$', views.objectview, name="objectview"),
    re_path(r'^object/(?P<container>.+?)/(?P<objectname>.+?)$', views.object_item, name="object"),
    re_path(r'^upload/(?P<container>.+?)/(?P<prefix>.+)?$', views.upload, name="upload"),
    re_path(r'^create_object/(?P<container>.+?)/(?P<prefix>.+)?$', views.create_object, name="create_object"),
    re_path(r'^create_pseudofolder/(?P<container>.+?)/(?P<prefix>.+)?$', views.create_pseudofolder, name="create_pseudofolder"),
    re_path(r'^delete_container/(?P<container>.+?)$', views.delete_container_view, name="delete_container"),
    re_path(r'^delete/(?P<container>.+?)/(?P<objectname>.+?)$', views.delete_object_view, name="delete_object"),
    re_path(r'^delete_pseudofolder/(?P<container>.+?)/(?P<pseudofolder>.+?)$', views.delete_pseudofolder, name="delete_pseudofolder"),
    re_path(r'^create_container/?$', views.create_container, name="create_container"),
    re_path(r'^acls/(?P<container>.+?)/$', views.edit_acl, name="edit_acl"),
    re_path(r'^cors/(?P<container>.+?)/$', views.edit_cors, name="edit_cors"),
    re_path(r'^download/(?P<container>.+?)/(?P<objectname>.+?)$', views.download, name="download"),
    re_path(r'^preview/(?P<container>.+?)/(?P<objectname>.+?)$', views.download, name="preview"),
    re_path(r'^metadata/(?P<container>.+?)/(?P<objectname>.+?)?$', views.metadataview, name="metadata"),
    re_path(r'^custom-metadata/(?P<container>.+?)/(?P<objectname>.+?)?$', views.edit_custom_metadata, name="edit_custom_metadata"),
    re_path(r'^cache-control/(?P<container>.+?)/(?P<objectname>.+?)?$', views.cache_control, name="cache_control"),
    re_path(r'^versioning/(?P<container>.+?)/(?P<prefix>(.+)+)?$', views.object_versioning, name="object_versioning"),
    re_path(r'^optional-headers/(?P<container>.+?)/(?P<objectname>.+?)?$', views.optional_headers, name="optional_headers"),
    # API
    re_path(r'^api/info$', views.info_json, name="info_json"),
    re_path(r'^api/backup-list/?', views.container_backup_list, name="container_backup_list"),

    # Storage base url (/)
    re_path(r'^', views.containerview, name="containerview")
]



# -*- coding: utf-8 -*-

import logging

from datetime import datetime

from django import template
from django.utils.timezone import utc
from django.template.defaultfilters import stringfilter

from storage.views.backup import get_current_backup


register = template.Library()
log = logging.getLogger(__name__)


@register.inclusion_tag("storage/storage_tools.html", takes_context=True)
def storage_tools(context):
    meta = context.get('container_meta')
    container = context.get("container")
    project_id = context.get("project_id")
    project_name = context.get("project_name")

    trash_enabled = meta.get('x-undelete-enabled')
    if trash_enabled is None or trash_enabled == 'False':
        trash_enabled = False

    return {
        'trash_enabled': trash_enabled,
        'backup_obj': get_current_backup(container, project_id),
        'container': container,
        'prefix': context.get('prefix'),
        'project_id': project_id,
        'project_name': project_name
    }


@register.filter
@stringfilter
def lastpart(value):
    value = value.strip('/').split('/')[-1]
    return value


@register.filter
@stringfilter
def dateconv(value):
    try:
        value = datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
        value = value.replace(tzinfo=utc)
    except ValueError:
        value = 0.0
    return value



# -*- coding: utf-8 -*-

""" Forms for storage.browser """

from django import forms
from django.core import validators


class CreateContainerForm(forms.Form):
    """ Simple form for container creation """
    containername = forms.CharField(label='Container Name', max_length=100, required=True,
        validators=[validators.validate_slug],
        widget=forms.TextInput(attrs={'class': 'form-control'}),
        error_messages={'invalid': 'Enter a valid name consisting of letters, numbers, underscores or hyphens.'})


class AddACLForm(forms.Form):
    """ Form for ACLs """
    username = forms.CharField(max_length=100)
    read = forms.BooleanField(required=False)
    write = forms.BooleanField(required=False)


class AddCORSForm(forms.Form):
    """ Form for CORS """
    host = forms.CharField(max_length=100)


class PseudoFolderForm(forms.Form):
    """ Upload form """
    foldername = forms.CharField(label='Pseudofolder Name', max_length=100, required=True,
        widget=forms.TextInput(attrs={'class': 'form-control'}))


class RemoveCacheForm(forms.Form):
    """ Upload form """
    urls = forms.CharField(label='URLs', required=True,
        widget=forms.Textarea({'cols': 100}))



# -*- coding:utf-8 -*-

from unittest.mock import Mock, patch
from unittest import TestCase

from django.utils.translation import gettext as _

from keystoneclient import exceptions

from identity import views
from identity.tests.fakes import FakeResource

from vault.tests.fakes import fake_request


class ListProjectTest(TestCase):

    def setUp(self):
        patch('identity.keystone.Keystone._create_keystone_connection').start()
        patch('identity.views.log').start()

        self.view = views.ListProjectView.as_view()
        self.request = fake_request(method='GET')

    def tearDown(self):
        patch.stopall()

    def test_list_projects_needs_authentication(self):
        req = fake_request(method='GET', user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_show_project_list(self):
        """
        We only check if the project_list view is being called with the
        right list of projects on the context.
        """
        fake_project = FakeResource(1)

        # This mocks is faking keystone retrieving a defined list of
        # projects
        patch('identity.views.Keystone.project_list',
              Mock(return_value=[fake_project])).start()

        render_mock = patch(
            'identity.views.ListProjectView.render_to_response').start()

        response = self.view(self.request)

        render_args = render_mock.call_args[0][0]
        computed = render_args['projects'][0]

        self.assertEqual(computed, fake_project.to_dict())

    @patch('identity.keystone.Keystone.project_list')
    def test_list_project_view_exception(self, mock_project_list):
        mock_project_list.side_effect = Exception()

        response = self.view(self.request)
        msgs = [msg for msg in self.request._messages]

        self.assertGreater(len(msgs), 0)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(msgs[0].message, _('Unable to list projects'))


class CreateProjectTest(TestCase):

    def setUp(self):
        patch('identity.keystone.Keystone._create_keystone_connection').start()
        patch('identity.views.actionlog.log').start()
        patch('identity.views.log').start()
        patch('identity.views.CreateProjectView.get_context_data').start()

        self.form = patch('identity.views.ProjectForm').start()
        self.view = views.CreateProjectView.as_view()

        self.request = fake_request(method='POST')

        self.group = self.request.user.groups.first()

        post = self.request.POST.copy()
        post.update({
            'name': 'project-teste',
            'description': 'desc',
            'enabled': True,
            'group': self.group.id
        })
        self.request.POST = post

    def test_create_project_needs_authentication(self):
        req = fake_request(method='POST', user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    @patch('identity.keystone.Keystone.vault_project_create')
    def test_invove_vault_project_create_when_request_is_valid(self, mock):
        mock.return_value = {
            'status': True,
            'user': FakeResource(),
            'project': FakeResource()
        }

        # Fake the validation of the form
        self.form.return_value.is_valid.return_value = True
        response = self.view(self.request)
        group_id = self.group.id

        mock.assert_called_with('project-teste', group_id,
                                created_by='vault', description='desc',
                                team_owner_id=group_id, first_team_id=group_id)

    @patch('identity.keystone.Keystone.vault_project_create')
    def test_dont_invove_vault_project_create_when_request_is_valid(self, mock):

        # Fake the validation of the form
        self.form.return_value.is_valid.return_value = False
        response = self.view(self.request)

        self.assertFalse(mock.called)


class CreateProjectSuccessTest(TestCase):

    def setUp(self):
        self.view = views.CreateProjectSuccessView.as_view()
        self.request = fake_request(method='GET')
        self.mock_keystone_conn = patch('identity.keystone.Keystone._create_keystone_connection').start()

    @patch('identity.views.Keystone.get_endpoints')
    @patch('identity.views.CreateProjectSuccessView.render_to_response')
    def test_render_success_create_page(self, mock_render, mock_get_endpoints):

        os_endpoints = {
            u'adminURL': u'https://adminURL/storage',
            u'publicURL': u'https://publicURL/storage',
            u'internalURL': u'https://internalURL/storage'}

        bs_enpoints = {
            u'adminURL': u'https://adminURL/faas',
            u'publicURL': u'https://publicURL/faas',
            u'internalURL': u'https://internalURL/faas'}

        mock_get_endpoints.return_value = {
            u'object_store': os_endpoints,
            u'block_storage': bs_enpoints
        }

        project_create_result = {
            'user_name': 'fake_user',
            'project_name': 'fake_project',
            'user_password': 'secret'
        }

        self.request.session['project_info'] = project_create_result
        _ = self.view(self.request)

        called_args, _ = mock_render.call_args
        computed_project_info = called_args[0]['project_info']

        computed_os_endpoints = computed_project_info['endpoints']['object_store']
        self.assertEqual(os_endpoints, computed_os_endpoints)

        computed_bs_enpoints = computed_project_info['endpoints']['block_storage']
        self.assertEqual(bs_enpoints, computed_bs_enpoints)

        self.assertEqual('fake_user', computed_project_info['user_name'])
        self.assertEqual('fake_project', computed_project_info['project_name'])
        self.assertEqual('secret', computed_project_info['user_password'])


class UpdateProjectTestTest(TestCase):
    """
    Tests for UpdateProjectView

    - Verify if the form is invalid, won't invoke
      keystone.vault_update_project_tsuru

    - Verify if the form is valid, what arguments are being used
    """

    def setUp(self):
        patch('identity.keystone.Keystone._create_keystone_connection').start()
        patch('identity.views.actionlog.log').start()
        patch('identity.views.log').start()

        patch('identity.views.Keystone.project_get',
              return_value=FakeResource(1, 'fake_resource')).start()

        self.mock_get_context_data = patch(
            'identity.views.UpdateProjectView.get_context_data').start()

        # Mock form validation. It's mocked as a invalid form.
        form = patch('identity.views.ProjectForm').start()
        self.form_is_valid = form.return_value.is_valid

        self.view = views.UpdateProjectView.as_view()

        self.request = fake_request(method='POST')

        post = self.request.POST.copy()
        post.update({
            'name': 'update_teste',
            'description': 'update desc',
            'group': 1,
            'enabled': 'False'})
        self.request.POST = post

    @patch('identity.keystone.Keystone.vault_project_update')
    def test_invalid_form_dont_invoke_vault_project_update(self, mock):
        # Inverting this return value must fail the test
        self.form_is_valid.return_value = False
        _ = self.view(self.request)

        mock.assert_not_called()
        self.assertTrue(self.mock_get_context_data.called)

    @patch('identity.keystone.Keystone.vault_project_update')
    def test_valid_form_invoke_vault_project_update(self, mock):
        # Inverting this return value must fail the test
        self.form_is_valid.return_value = True
        _ = self.view(self.request)

        group_id = self.request.POST.get('group')
        mock.assert_called_with(
            1,
            'fake_resource',
            group_id,
            description=self.request.POST.get('description'),
            enabled=False,
            team_owner_id=group_id)


class DeleteProjectTest(TestCase):
    """
    Tests for DeleteProjectView

    - Verify if the form is invalid, won't invoke
      keystone.vault_delete_project

    - Verify if the form is valid, but the credentials are invalid,
      won't invoke keystone.vault_delete_project

    - Verify if the form and the credentials are valid, what arguments
      are being used to invoke keystone.vault_delete_project
    """

    def setUp(self):
        patch('identity.keystone.Keystone._create_keystone_connection').start()
        patch('identity.views.actionlog.log').start()
        patch('identity.views.log').start()
        patch('identity.views.DeleteProjectView.get_context_data').start()

        self.form = patch('identity.views.DeleteProjectConfirm').start()
        self.view = views.DeleteProjectView.as_view()
        self.request = fake_request(method='GET')

    def tearDown(self):
        patch.stopall()

    def test_delete_project_needs_authentication(self):
        req = fake_request(method='GET', user=False)
        response = self.view(req)

        self.assertEqual(response.status_code, 302)

    def test_get_request_invoke_render_with_form_object(self):
        """
        It tests if the view render method is invoked passing the proper
        form object
        """

        mock_render = patch(
            'identity.views.DeleteProjectView.get_context_data').start()

        fake_resource = FakeResource()
        self.form.return_value = fake_resource
        _ = self.view(self.request)

        computed_form = mock_render.call_args[1].get('form')
        self.assertEqual(fake_resource, computed_form)

    def test_post_invalid_form(self):
        mock_delete = patch(
            'identity.keystone.Keystone.vault_project_delete').start()

        mock_render = patch(
            'identity.views.DeleteProjectView.render_to_response').start()

        self.request.method = 'POST'

        self.form.return_value.is_valid.return_value = False

        _ = self.view(self.request)

        self.assertEqual(1, mock_render.call_count)
        mock_delete.assert_not_called()

    @patch('identity.views.Keystone')
    def test_get_delete_url_return_200(self, mock_keystone):
        self.request.method = 'GET'
        response = self.view(self.request)

        self.assertEqual(200, response.status_code)

    @patch('identity.views.Keystone')
    def test_delete_project_connect_with_username_and_password_passed_by_form(self, mock_keystone):
        self.view(self.request, project_id='12345abcef')

        mock_keystone.asser_called_with(
            self.request, username='teste_user', password='secret')



# -*- coding: utf-8 -*-
"""
Django settings for vault project.
"""

import os
import importlib
from django.utils.translation import gettext_lazy as _


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = os.getenv("VAULT_SECRET_KEY", "vaultsecretkey")

PROJECT = "vault"

DEBUG = os.getenv("VAULT_DEBUG", "True") == "True"

ALLOWED_HOSTS = ["*"]

# Disable HTTPS verification warnings in debug mode
if DEBUG:
    from requests.packages import urllib3
    urllib3.disable_warnings()

ENVIRON = os.getenv("VAULT_ENVIRON", None)

INSTALLED_APPS = [
    "allaccess",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "vault",
    "actionlogger",
    "storage",
    "identity",
]

AUTHENTICATION_BACKENDS = (
    "django.contrib.auth.backends.ModelBackend",
    "allaccess.backends.AuthorizedServiceBackend",
)

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "vault.context_processors.vault_settings",
                "vault.context_processors.vault_session",
            ],
        },
    },
]

WSGI_APPLICATION = "vault.wsgi.application"

ROOT_URLCONF = "vault.urls"

LOGIN_URL = "/admin/login/"

# Password validation
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME":
        "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {
        "NAME":
        "django.contrib.auth.password_validation.MinimumLengthValidator"
    },
    {
        "NAME":
        "django.contrib.auth.password_validation.CommonPasswordValidator"
    },
    {
        "NAME":
        "django.contrib.auth.password_validation.NumericPasswordValidator"
    },
]

# Internationalization
LANGUAGE_CODE = os.getenv("VAULT_LANGUAGE", "en-us")
LANGUAGES = (
    ("en", _("English")),
    ("pt-BR", _("Portuguese")),
)

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
STATIC_ROOT = "vault_static/"

STATIC_URL = "{}/{}/{}".format(os.getenv("SWIFT_INTERNAL_URL", ""),
                               os.getenv("SWIFT_CONTAINER", "vault"),
                               STATIC_ROOT)

# The openstack_auth.user.Token object isn't JSON-serializable ATM
SESSION_SERIALIZER = "django.contrib.sessions.serializers.PickleSerializer"

PAGINATION_SIZE = int(os.getenv("VAULT_PAGINATION_SIZE", 50))

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.mysql",
        "NAME": os.getenv("VAULT_MYSQL_DB", PROJECT),
        "USER": os.getenv("VAULT_MYSQL_USER", "root"),
        "PASSWORD": os.getenv("VAULT_MYSQL_PASSWORD", ""),
        "HOST": os.getenv("VAULT_MYSQL_HOST", "127.0.0.1"),
        "PORT": int(os.getenv("VAULT_MYSQL_PORT", 3306)),
        "OPTIONS": {
            "init_command": "SET sql_mode='STRICT_TRANS_TABLES'",
        },
    }
}

MAX_FILES_UPLOAD = os.getenv("MAX_FILES_UPLOAD", 10)

# Help url
HELP_URL = os.getenv("VAULT_HELP_URL")

# Proxy
VAULT_HTTPS_PROXY = os.getenv("VAULT_HTTPS_PROXY")
VAULT_HTTP_PROXY = os.getenv("VAULT_HTTP_PROXY")

# When versioning is enabled in a container named <container>, another
# container named <prefix><container> will be created to keep objects versions
SWIFT_VERSION_PREFIX = os.getenv("VAULT_SWIFT_VERSION_PREFIX", "_version_")
SWIFT_TRASH_PREFIX = ".trash"
SWIFT_HIDE_PREFIXES = [SWIFT_VERSION_PREFIX, SWIFT_TRASH_PREFIX]

# Backup
BACKUP_ENABLED = os.getenv("VAULT_BACKUP_ENABLED", "False") == "True"
BACKUP_USER = os.getenv("VAULT_BACKUP_USER")
BACKUP_USER_ROLE = os.getenv("VAULT_BACKUP_USER_ROLE")
BACKUP_API_URL = os.getenv("VAULT_BACKUP_API_URL")
BACKUP_API_USER = os.getenv("VAULT_BACKUP_API_USER")
BACKUP_API_PASSWORD = os.getenv("VAULT_BACKUP_API_PASSWORD")
BACKUP_OBJECT_COUNT_VALUE = os.getenv("BACKUP_OBJECT_COUNT_VALUE", "10000")
BACKUP_OBJECT_BYTES_VALUE = os.getenv("BACKUP_OBJECT_BYTES_VALUE",
                                      "1000000000")

# True if you are using invalid SSL certs
SWIFT_INSECURE = os.getenv("VAULT_SWIFT_INSECURE", "False") == "True"

# Timeout for requests made with swiftclient
SWIFT_REQUESTS_TIMEOUT = os.getenv("VAULT_SWIFT_REQUESTS_TIMEOUT", 60)

# Keystone
KEYSTONE_USERNAME = os.getenv("VAULT_KEYSTONE_USERNAME", "u_vault")
KEYSTONE_PASSWORD = os.getenv("VAULT_KEYSTONE_PASSWORD", "u_vault")
KEYSTONE_PROJECT = os.getenv("VAULT_KEYSTONE_PROJECT", "Vault")
KEYSTONE_VERSION = os.getenv("VAULT_KEYSTONE_API_VERSION", 3)
KEYSTONE_URL = os.getenv(
    "VAULT_KEYSTONE_URL", "http://localhost:5000/{}".format(
        "v2.0" if KEYSTONE_VERSION == 2 else "v3"))

KEYSTONE_TIMEOUT = os.getenv("VAULT_KEYSTONE_TIMEOUT", 3)
# KEYSTONE_INSECURE = False
KEYSTONE_ROLE = os.getenv("VAULT_KEYSTONE_ROLE")  # swiftoperator role ID

# Cache
CACHE_APIS_ENABLED = os.getenv("CACHE_APIS_ENABLED", "False") == "True"
CACHESWEEP_API = os.getenv("CACHESWEEP_API", "")
PRE_CACHE_API = os.getenv("PRE_CACHE_API", "")

IDENTITY_SECRET_KEY = os.getenv("IDENTITY_SECRET_KEY")

if not IDENTITY_SECRET_KEY and DEBUG:
    IDENTITY_SECRET_KEY = "SRTAYZDUnNSW9xhBStvylCgmCY5jo3zedqDNbdRN3Ek="

# Swift-Cloud
SWIFT_CLOUD_ENABLED = os.getenv("VAULT_SWIFT_CLOUD_ENABLED", "False") == "True"
SWIFT_CLOUD_TOOLS_URL = os.getenv("VAULT_SWIFT_CLOUD_TOOLS_URL")
SWIFT_CLOUD_TOOLS_API_KEY = os.getenv("VAULT_SWIFT_CLOUD_TOOLS_API_KEY")

# Logging conf
LOGGING = {
    "version": 1,
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
        },
    },
    "loggers": {
        "identity": {
            "handlers": ["console"],
            "propagate": True,
            "level": "DEBUG",
        },
        "vault": {
            "handlers": ["console"],
            "propagate": True,
            "level": "DEBUG",
        },
        "storage": {
            "handlers": ["console"],
            "propagate": True,
            "level": "DEBUG",
        },
        "dashboard": {
            "handlers": ["console"],
            "propagate": True,
            "level": "DEBUG",
        },
    },
}


def load_apps_settings():
    """Load Apps' settings"""

    new_settings = {}
    for app in INSTALLED_APPS:
        try:
            module = importlib.import_module(app)
            module_apps = importlib.import_module(app + ".apps")
            app_config = getattr(module_apps,
                                 module.default_app_config.split(".")[-1])
            if app_config.vault_app is True:
                app_settings = importlib.import_module(app + ".settings")
                names = [
                    x for x in app_settings.__dict__ if not x.startswith("_")
                ]
                new_settings.update(
                    {k: getattr(app_settings, k)
                     for k in names})
        # Ignores apps that don't have a "vault_app = True" in their apps.py file
        # or don't have a settings.py file
        except (AttributeError, ModuleNotFoundError) as e:
            pass
    return new_settings


globals().update(load_apps_settings())



# -*- coding: utf-8 -*-

import factory
from datetime import datetime

from django.contrib.auth.models import User, Group

from vault.models import GroupProjects


class FakeResource:
    """ Fake Keystone Resource (e.g. User, Project, Role) """
    def __init__(self, n=0, name=None):
        self.id = n
        self.name = name if name else "FakeResource{:d}".format(n)
        self.enabled = True
        self.description = ''
        self.project_id = 1

    def to_dict(self):
        return self.__dict__


class FakeToken:
    """ Fake Keystone Token """
    def __init__(self, id="faketokenid"):
        self.id = id


class FakeKeystone:

    def __init__(self, request):
        self.users = FakeResource(1)
        self.roles = FakeResource(1)
        self.tenants = FakeResource(1)
        self.projects = FakeResource(1)

        self.users.get = lambda a: request.user
        self.users.create = lambda name, password, email, project, enabled, domain=None: None
        self.users.update = lambda user, **kwargs: None
        self.roles.get = lambda id: FakeResource(id)
        self.tenants.get = lambda a: FakeResource(1)
        self.tenants.create = lambda name, domain_id='default', description=None, enabled=True: None
        self.projects.get = lambda a: FakeResource(1)


# factories.py
class GroupFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Group
        django_get_or_create = ('name',)
        # strategy = factory.BUILD_STRATEGY

    # id = factory.Sequence(lambda n: n)
    name = factory.Sequence(lambda n: "Group #{}".format(n))


class UserFactory(factory.django.DjangoModelFactory):

    class Meta:
        model = User
        django_get_or_create = ('first_name', 'last_name', 'is_superuser',)
        # strategy = factory.BUILD_STRATEGY

    pk = 1
    first_name = "John"
    last_name = "Doe"
    is_superuser = True

    @factory.post_generation
    def groups(self, create, extracted, **kwargs):
        if not create:
            # Simple build, do nothing.
            return

        if extracted:
            # A list of groups were passed in, use them
            for group in extracted:
                self.groups.add(group)


class GroupProjectsFactory(factory.django.DjangoModelFactory):

    class Meta:
        model = GroupProjects
        # strategy = factory.BUILD_STRATEGY



# -*- coding: utf-8 -*-

from django.contrib import admin
from .models import Audit


class AuditAdmin(admin.ModelAdmin):
    search_fields = ['user', 'action', 'item', 'through']


admin.site.register(Audit, AuditAdmin)



# -*- coding: utf-8 -*-

from django.db import models


class Project(models.Model):
    id = models.AutoField(primary_key=True)
    project = models.CharField(max_length=255)
    user = models.CharField(max_length=255)
    password = models.CharField(max_length=255)
    created_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = 'project'
        unique_together = (('project'),)



# -*- coding: utf-8 -*-

import os
from io import StringIO

from django.contrib.auth.models import AnonymousUser
from django.core.files.uploadedfile import InMemoryUploadedFile


# From: http://www.rkblog.rk.edu.pl/w/p/temporary-files-django-tests-and-fly-file-manipulation/
def get_temporary_text_file():
    io = StringIO()
    io.write('foo')
    io.seek(0, os.SEEK_END)
    text_file = InMemoryUploadedFile(io, None, 'foo.txt', 'text', io.tell(), None)
    text_file.seek(0)
    return text_file


def get_account():
    account_stat = {
        'content-length': '147',
        'x-account-object-count': '5',
        'server': 'nginx',
        'connection': 'keep-alive',
        'x-timestamp': '1405090896.54396',
        'x-account-meta-temp-url-key': 'ipytkrl2sij2gbycz10xrem23hfsz3z4',
        'x-trans-id': 'tx11e34a0efd904a7cbd6e7-0053f49d36',
        'date': 'Wed, 20 Aug 2014 13:05:58 GMT',
        'x-account-bytes-used': '46',
        'x-account-container-count': '3',
        'content-type': 'application/json; charset=utf-8',
        'accept-ranges': 'bytes'
    }

    containers = [
        {'count': 4, 'bytes': 32, 'name': 'container1'},
        {'count': 0, 'bytes': 0, 'name': 'container2'},
        {'count': 1, 'bytes': 14, 'name': 'container3'}
    ]

    return (account_stat, containers)


def get_container(*args, **kwargs):
    objects = [{'bytes': 8, 'last_modified': '2014-07-11T18:51:45.260310',
                'name': 'obj_pelo_vip.html', 'content_type': 'text/html'},

               {'bytes': 8, 'last_modified': '2014-07-11T17:01:35.526450',
                'hash': 'bdf0a1cfded326aeb439fa86bd3427ab', 'name': 'ok',
                'content_type': 'application/octet-stream'},

               {'bytes': 0, 'last_modified': '2014-07-11T17:01:35.526450',
                'hash': 'bdf0a1cfded326aeb439fa86bd3427ab',
                'name': 'pseudofolder/', 'content_type': 'application/directory'}]

    if kwargs.get('trash_container'):
        objects = [{'bytes': 8, 'last_modified': '2014-07-11T18:52:45.260310',
                    'name': 'obj_pelo_vip_trash.html', 'content_type': 'text/html'}]

    return ({}, objects)


def get_containers(objects):
    self.headers = {'Content-Length': '7', 'X-Container-Object-Count': '1', 'Accept-Ranges': 'bytes', 'Vary': 'Origin', 'Server': 'nginx', 'X-Container-Bytes-Used': '0', 'Connection': 'keep-alive', 'X-Timestamp': '1465405001.73901', 'X-Trans-Id': 'tx52fbf897372b486ebd33a-005762b25d', 'Date': 'Thu, 16 Jun 2016 14:06:21 GMT', 'X-Storage-Policy': 'default-3x', 'Content-Type': 'text/plain; charset=utf-8'}

    return ({}, objects)


class FakeResource:
    """Fake Keystone Resource (e.g. User, Project, Role)"""

    def __init__(self, n=0, name=None):
        self.id = n
        self.name = name if name else "FakeResource{:d}".format(n)
        self.enabled = True
        self.description = ''
        self.project_id = 1


class FakeToken:
    """ Fake Keystone Token """

    def __init__(self, id="faketokenid"):
        self.id = id


class FakeUser(AnonymousUser):
    """Fake Keystone Resource (e.g. User, Project, Role)"""

    def __init__(self, n=0, name=None):
        super(FakeUser, self).__init__()

        self.id = n
        self.username = name if name else "FakeUser{:d}".format(n)
        self.enabled = True
        self.description = ''


class FakeRequestResponse:

    def __init__(self, status_code=200, content=None, headers=None):
        self.status_code = status_code
        self.content = content
        self.headers = headers


class FakeElasticResult:
    """Fake Elastic Result"""

    def __init__(self, headers):
        self.headers = headers



# -*- coding: utf-8 -*-

import logging
from django import forms
from django.forms.widgets import Select
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.contrib.auth.models import Group, User
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import gettext_lazy as _

from keystoneclient import exceptions

from vault.models import GroupProjects
from identity.keystone import KeystoneNoRequest


log = logging.getLogger(__name__)


class VaultLoginForm(AuthenticationForm):

    def __init__(self, *args, **kwargs):
        super(VaultLoginForm, self).__init__(*args, **kwargs)

    username = forms.CharField(widget=forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Username'}))
    password = forms.CharField(widget=forms.PasswordInput(attrs={'class': 'form-control', 'placeholder': 'Password'}))


class GroupAdminForm(forms.ModelForm):

    class Meta:
        model = Group
        exclude = []

    choices = ((0, ''),)

    users = forms.ModelMultipleChoiceField(
        queryset=User.objects.all(),
        required=False,
        widget=FilteredSelectMultiple('users', False)
    )

    def __init__(self, *args, **kwargs):
        super(GroupAdminForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['users'].initial = self.instance.user_set.all()

    def save_m2m(self):
        self.instance.user_set.set(self.cleaned_data['users'])

    def save(self, *args, **kwargs):
        instance = super(GroupAdminForm, self).save()
        return instance


def get_project_choices():
    try:
        keystone = KeystoneNoRequest()
    except exceptions.AuthorizationFailure:
        msg = _('Unable to retrieve Keystone data')
        # messages.add_message(request, messages.ERROR, msg)
        log.error(f'In get_project_choices(): {msg}')

        return tuple([(u'', u'---------')])

    project_list = []

    if keystone.conn is not None:
        for project in keystone.project_list():
            project_list.append((project.id, project.name))

    project_list.sort(key=lambda x: x[1].lower())
    project_list = [(u'', u'---------')] + project_list

    return tuple(project_list)


class GroupProjectsForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(GroupProjectsForm, self).__init__(*args, **kwargs)
        self.fields['project'] = forms.ChoiceField(choices=get_project_choices(), required=True)

    class Meta:
        model = GroupProjects
        fields = ['group', 'project']



# -*- coding: utf-8 -*-

import os
import logging

from datetime import datetime, timedelta
from cryptography.fernet import Fernet

from swiftclient import client

from django.conf import settings
from django.contrib import messages
from django.shortcuts import render
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

from keystoneclient import exceptions

from vault.models import GroupProjects, CurrentProject
from identity.keystone import Keystone, KeystoneNoRequest

log = logging.getLogger(__name__)


def update_default_context(request, context={}):
    if not request.session.get('is_superuser'):
        request.session['is_superuser'] = request.user.is_superuser

    context.update({
        'logged_user': request.user,
        'project_id': request.session.get('project_id'),
        'project_name': request.session.get('project_name'),
        'is_superuser': request.user.is_superuser
    })

    return context


def generic_pagination(items,
                       page=1,
                       per_page=settings.PAGINATION_SIZE,
                       query=None):
    paginator = Paginator(items, per_page)

    try:
        paginated_items = paginator.page(page)
        if (query):
            paginated_items.query = query
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        paginated_items = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        paginated_items = paginator.page(paginator.num_pages)

    return paginated_items


def save_current_project(user_id, project_id):
    """Save on database user current project"""

    try:
        current = CurrentProject.objects.get(user_id=user_id)
    except CurrentProject.DoesNotExist:
        current = CurrentProject()
        current.user_id = user_id

    current.project = project_id
    current.save()

    return current


def set_current_project(request, project):
    """Set current project on django session"""

    if project is not None:
        request.session['project_id'] = project.id
        request.session['project_name'] = project.name
    else:
        # Project doesn't exist
        request.session.pop('project_id', None)
        request.session.pop('project_name', None)

    return maybe_update_token(request)


def maybe_update_token(request):
    token_time = request.session.get('token_time')

    if token_time is None or token_time < datetime.utcnow():
        log.info('Updating token for user [{}]'.format(request.user))

        try:
            keystone = Keystone(request)
        except exceptions.AuthorizationFailure:
            msg = _('Unable to retrieve Keystone data')
            messages.add_message(request, messages.ERROR, msg)
            log.error(f'{request.user}: {msg}')

            return False

        if keystone.conn is None:
            return False

        request.session['token_time'] = (timedelta(minutes=15) +
                                         datetime.utcnow())
        request.session['auth_token'] = keystone.conn.auth_token
        request.session['service_catalog'] = keystone.get_endpoints()

        return True

    return False


def get_current_project(user_id):
    """Return from database user current project"""

    try:
        current = CurrentProject.objects.get(user_id=user_id)
    except CurrentProject.DoesNotExist:
        return None

    keystone = KeystoneNoRequest()
    return keystone.project_get(current.project)


def delete_current_project(user_id):
    """Delete from database user current project and removes it from session"""

    try:
        current = CurrentProject.objects.get(user_id=user_id)
        current.delete()
    except Exception:
        return False

    request.session.pop('project_id')
    request.session.pop('project_name')

    return True


def purge_current_project(request, project_id):
    """Delete all entries for project_id and removes it from session"""

    try:
        items = CurrentProject.objects.filter(project=project_id)
        items.delete()
    except Exception:
        return False

    request.session.pop('project_id')
    request.session.pop('project_name')

    return True


def project_check(request, current_project):
    project_id = request.session.get('project_id')
    user = request.user

    if current_project:
        try:
            keystone = Keystone(request)
        except exceptions.AuthorizationFailure:
            msg = _('Unable to retrieve Keystone data')
            messages.add_message(request, messages.ERROR, msg)
            log.error(f'{request.user}: {msg}')

            return False

        if not keystone.conn:
            return False

        project = keystone.project_get_by_name(current_project)

        if not project:
            # messages.add_message(request, messages.WARNING, u"Unauthorized")
            return False

        if not project_id or project.id != project_id:
            groups = user.groups.all()
            group_projects = GroupProjects.objects.filter(
                group_id__in=[group.id for group in groups])

            if group_projects.filter(project=project.id).count() == 0:
                # messages.add_message(request, messages.WARNING, u"Unauthorized")
                return False

            save_current_project(user.id, project.id)
            set_current_project(request, project)

    return True


def project_error(request, *args, **kwargs):
    context = {
        "project_name": request.session.get('project_name'),
        "inexistent_project": kwargs.get("inexistent_project"),
        "not_owned_project": kwargs.get("not_owned_project")
    }

    return render(request, "vault/project_error.html", context)


def project_required(view_func):
    def _wrapper(request, *args, **kwargs):
        current_project = kwargs.get('project')

        check = project_check(request, current_project)
        if not check:
            return project_error(request, inexistent_project=kwargs['project'])

        maybe_update_token(request)

        return view_func(request, *args, **kwargs)

    return _wrapper


def human_readable(value):
    """
    Returns the number in a human readable format; for example 1048576 = "1Mi".
    """
    value = float(value)
    index = -1
    suffixes = 'KMGTPEZY'
    while value >= 1024 and index + 1 < len(suffixes):
        index += 1
        value = round(value / 1024)
    if index == -1:
        return '{:d}'.format(value)
    return '{:d}{}i'.format(round(value), suffixes[index])


def encrypt_password(password):
    key = bytes(settings.IDENTITY_SECRET_KEY, encoding='utf8')
    cipher_suite = Fernet(key)
    return cipher_suite.encrypt(bytes(password, encoding='utf8'))


def decrypt_password(password):
    key = bytes(settings.IDENTITY_SECRET_KEY, encoding='utf8')
    cipher_suite = Fernet(key)
    return cipher_suite.decrypt(bytes(password, encoding='utf8'))



# -*- coding: utf-8 -*-
# pylint:disable=E0611, E1101

""" Standalone webinterface for Openstack Swift. """

import re
import string
import random
import logging

from urllib.parse import urlparse

from django.conf import settings
from django.contrib import messages
from django.utils.translation import gettext

from swiftclient import client
from identity.keystone import KeystoneNoRequest

log = logging.getLogger(__name__)


def get_storage_endpoint(request, endpoint_type):
    """
    service_catalog na sessao eh referente a conexao com o project "admin"
    para retornar o endpoint do "project_id" da sessao, usa-se o service_catalog
    do project "admin", substituindo o project_id

    Isto eh necessario pois a conexao com o Keystone eh sempre com o mesmo
    project, nao estando disponivel de forma direta o service_catalog de um
    project qualquer

    :param request: http resquest
    :param endpoint_type: adminURL, publicURL, internalURL
    :return: endpoint url or None

    """
    service_catalog = request.session.get('service_catalog')
    project_id = request.session.get('project_id')

    if service_catalog is None or project_id is None:
        msg = gettext('No project selected')
        log.error(msg)
        messages.add_message(request, messages.ERROR, msg)
        return None

    service = service_catalog.get('object_store')
    endpoint_on_session = service.get(endpoint_type)

    if endpoint_on_session is None:
        return None

    _, project_admin_id = endpoint_on_session.split('AUTH_')

    final_url = endpoint_on_session.replace(project_admin_id, project_id)

    return final_url


def get_token_id(request):
    return request.session.get('auth_token')


def replace_hyphens(olddict):
    """ Replaces all hyphens in dict keys with an underscore.

    Needed in Django templates to get a value from a dict by key name. """
    newdict = {}
    for key, value in olddict.items():
        key = key.replace('-', '_')
        newdict[key] = value
    return newdict


def prefix_list(prefix):
    prefixes = []

    if prefix:
        elements = prefix.split('/')
        elements = filter(None, elements)
        prefix = ""
        for element in elements:
            prefix += element + '/'
            prefixes.append({'display_name': element, 'full_name': prefix})

    return prefixes


def pseudofolder_object_list(objects, prefix, public_url):
    pseudofolders = []
    objs = []
    duplist = []

    for obj in objects:
        # Rackspace Cloudfiles uses application/directory
        # Cyberduck uses application/x-directory
        if obj.get('content_type', None) in ('application/directory',
                                             'application/x-directory'):
            obj['subdir'] = obj['name']

        if 'subdir' in obj:
            # make sure that there is a single slash at the end
            # Cyberduck appends a slash to the name of a pseudofolder
            entry = obj['subdir'].strip('/') + '/'
            if entry != prefix and entry not in duplist:
                duplist.append(entry)
                # pseudofolders.append((entry, obj['subdir']))
                pseudofolders.append({'prefix': entry, 'name': obj['subdir']})
        else:
            # verifico se e um internal folder
            # se for, preciso redirecionar pra i.s3
            pattern = re.compile('^internal_.+')
            if pattern.match(public_url.split('/')[-1]):
                public_url = public_url.replace('://s3.', '://i.s3.')
            obj['public_url'] = public_url + '/' + obj['name']
            objs.append(obj)

    # return (pseudofolders, objs)
    return pseudofolders + objs


def get_temp_key(storage_url, auth_token, http_conn):
    """ Tries to get meta-temp-url key from account.
    If not set, generate tempurl and save it to account.
    This requires at least account owner rights. """

    try:
        account = client.get_account(storage_url, auth_token,
                                     http_conn=http_conn)
    except client.ClientException:
        return None

    key = account[0].get('x-account-meta-temp-url-key')

    if not key:
        chars = string.ascii_lowercase + string.digits
        key = ''.join(random.choice(chars) for x in range(32))
        headers = {'x-account-meta-temp-url-key': key}

        try:
            client.post_account(storage_url, token=auth_token, headers=headers,
                                http_conn=http_conn)
        except client.ClientException:
            return None
    return key


def get_acls(storage_url, auth_token, container, http_conn):
    """ Returns ACLs of given container. """
    acls = client.head_container(storage_url,
                                 auth_token,
                                 container,
                                 http_conn=http_conn)

    readers = acls.get('x-container-read', '')
    writers = acls.get('x-container-write', '')
    return (readers, writers)


def get_cors(storage_url, auth_token, container, http_conn):
    """ Returns CORS header of given container. """
    headers = client.head_container(storage_url,
                                    auth_token,
                                    container,
                                    http_conn=http_conn)

    cors = headers.get('x-container-meta-access-control-allow-origin', '')

    return cors


def remove_duplicates(text, separator):
    """ Removes possible duplicates from a separator-separated list. """
    entries = text.split(separator)
    cleaned_entries = list(set(entries))
    text = separator.join(cleaned_entries)
    return text


def remove_duplicates_from_acl(acls):
    """ Removes possible duplicates from a comma-separated list. """
    return remove_duplicates(acls, ',')


def remove_duplicates_from_cors(cors):
    """ Removes possible duplicates from a space-separated list. """
    return remove_duplicates(cors, ' ')


def get_account_containers(storage_url, auth_token):
    """ List all containers in an account"""
    container_list = []
    http_conn = client.http_connection(storage_url,
                                       insecure=settings.SWIFT_INSECURE)

    _, containers = client.get_account(storage_url, auth_token,
                                       http_conn=http_conn)

    for container in containers:
        container_list.append(container['name'])

    return container_list


def get_container_objects(container, storage_url, auth_token):
    object_list = []
    http_conn = client.http_connection(storage_url, insecure=settings.SWIFT_INSECURE)

    _, objects = client.get_container(storage_url, auth_token, container,
                                      http_conn=http_conn)

    for object in objects:
        object_list.append(object['name'])

    return object_list


def update_swift_account(user, password, project_name, headers):
    keystone = KeystoneNoRequest(user, password, project_name)
    endpoints = keystone.get_endpoints()
    storage_url = endpoints.get("object_store").get("adminURL")
    http_conn = client.http_connection(
        storage_url, insecure=settings.SWIFT_INSECURE)

    try:
        client.post_account(storage_url,
            keystone.conn.auth_token, headers, http_conn=http_conn)
    except client.ClientException as err:
        log.exception('Exception: {}'.format(err))
        return False

    return True


def delete_swift_account(storage_url, auth_token):
    insecure = settings.SWIFT_INSECURE

    try:
        # Criar container vazio para garantir que o account existe no swift
        http_conn = client.http_connection(storage_url, insecure=insecure)

        client.put_container(storage_url, auth_token, 'dummy_container',
            http_conn=http_conn)
        client.delete_container(storage_url, auth_token, 'dummy_container',
            http_conn=http_conn)
    except client.ClientException as err:
        log.exception('Fail to create container "dummy_container": {0}'.format(err))
        return False

    try:
        # Deletar o account
        url = urlparse(storage_url)
        domain = '{}://{}'.format(url.scheme, url.netloc)
        path = url.path

        http_conn = client.HTTPConnection(domain, insecure=insecure)
        headers = {'X-Auth-Token': auth_token}

        resp = http_conn.request('DELETE', path, headers=headers)

        if resp.status_code != 204:
            log.exception('Fail to delete account {}: status code {}'.format(
                storage_url, resp.status_code
            ))
            return False

    except client.ClientException as err:
        log.exception('Exception: {0}'.format(err))
        return False

    return True


def get_name_from_document(data):
    return data.get('_source').get('object')


def get_uri_object_from_document(data):
    return data.get('hits').get('hits')[0].get('_id')



import logging
import requests
from django.conf import settings
from django.shortcuts import render
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.utils.translation import gettext_lazy as _
from vault import utils
from actionlogger.actionlogger import ActionLogger
from storage.forms import RemoveCacheForm

log = logging.getLogger(__name__)
actionlog = ActionLogger()


@utils.project_required
@login_required
def storage_cache(request, project):
    context = {}
    return render(request, 'storage/cache.html', context)


@utils.project_required
@login_required
def remove_from_cache(request, project):
    form = RemoveCacheForm(request.POST or None)

    if form.is_valid():
        urls = form.cleaned_data['urls'].splitlines()

        data = {
            'url': urls,
            'user': request.user.username
        }

        api_url = '{host}/url/add'.format(host=settings.CACHESWEEP_API)
        req = requests.post(api_url, json=data)

        if req.status_code == 201:
            messages.add_message(request, messages.SUCCESS,
                _('URLs scheduled to be removed. Grab some coffee, it takes 2 to 3 minutes to remove.'))

            form = RemoveCacheForm(None)
            actionlog.log(request.user.username, "remove_cache", repr(urls))
        else:
            messages.add_message(request, messages.ERROR,
                _('Fail to schedule URLs to be removed.'))

    context = utils.update_default_context(request, {'form': form})

    return render(request, 'storage/cache_remove.html', context)


@utils.project_required
@login_required
def add_to_cache(request, project):
    context = {'pre_cache_api_url': ''}
    if hasattr(settings, 'PRE_CACHE_API'):
        context['pre_cache_api_url'] = settings.PRE_CACHE_API

    return render(request, 'storage/cache_add.html', context)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-12-03 22:53
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CurrentProject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('project', models.CharField(max_length=255)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'current_project',
                'verbose_name_plural': 'Current Project',
            },
        ),
        migrations.CreateModel(
            name='GroupProjects',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('project', models.CharField(max_length=255)),
                ('owner', models.BooleanField(default=0)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
            ],
            options={
                'db_table': 'group_projects',
                'verbose_name_plural': 'Groups and Projects',
            },
        ),
        migrations.AlterUniqueTogether(
            name='groupprojects',
            unique_together=set([('group', 'project')]),
        ),
    ]



# -*- coding: utf-8 -*-

import requests
from unittest.mock import patch
from unittest import TestCase

from swiftclient import client

from storage import utils

from vault.tests.fakes import fake_request
from storage.tests import fakes


class TestStorageUtils(TestCase):

    def test_remove_duplicates_from_acl(self):
        given = 'projectfake:userfake,projectfake2:userfake2,projectfake:userfake'
        computed = utils.remove_duplicates_from_acl(given)

        self.assertEqual(len(computed.split(',')), 2)

    def test_remove_duplicates_from_acl_empty_acls(self):
        given = ''
        expected = ''
        computed = utils.remove_duplicates_from_acl(given)

        self.assertEqual(expected, computed)

    def test_remove_duplicates_from_acl_no_duplicated_acls(self):
        given = 'projectfake:userfake,projectfake2:userfake2'
        computed = utils.remove_duplicates_from_acl(given)

        self.assertIn('projectfake:userfake', computed)
        self.assertIn('projectfake2:userfake2', computed)

    def test_replace_hyphens(self):
        fake_dict = {'content-length': '147', 'content-type': 'application/json'}
        expected = {'content_length': '147', 'content_type': 'application/json'}
        modified = utils.replace_hyphens(fake_dict)

        self.assertEqual(modified, expected)

    @patch('storage.utils.client.get_account')
    def test_get_temp_key(self, mock_get_account):
        expected_key = 'asgdia7239rgahsjbfhsd'
        mock_get_account.return_value = [
            {'x-account-meta-temp-url-key': expected_key}
        ]

        computed_key = utils.get_temp_key('http://fakeurl', 'faketoken', False)

        self.assertEqual(expected_key, computed_key)

    @patch('storage.utils.client.get_account')
    def test_get_temp_key_get_account_exception(self, mock_get_account):
        mock_get_account.side_effect = client.ClientException('')
        computed_key = utils.get_temp_key('http://fakeurl', 'faketoken', False)
        self.assertIsNone(computed_key)

    @patch('storage.utils.client.post_account')
    @patch('storage.utils.client.get_account')
    def test_get_temp_key_get_account_no_key_retrieved_on_get_account(self, mock_get_account, mock_post_account):
        mock_get_account.return_value = [{}]
        retrieved_key = utils.get_temp_key('http://fakeurl', 'faketoken', False)

        name, args, kargs = mock_post_account.mock_calls[0]
        computed_key = kargs['headers']['x-account-meta-temp-url-key']

        self.assertEqual(retrieved_key, computed_key)

    @patch('storage.utils.client.post_account')
    @patch('storage.utils.client.get_account')
    def test_get_temp_key_post_account_exception(self, mock_get_account, mock_post_account):
        mock_get_account.return_value = [{}]
        mock_post_account.side_effect = client.ClientException('')

        computed_key = utils.get_temp_key('http://fakeurl', 'faketoken', False)

        self.assertIsNone(computed_key)

    def test_get_storage_endpoint_urls(self):
        user = fakes.FakeUser(1, 'user')
        request = fake_request(user=user)

        admin_url = utils.get_storage_endpoint(request, 'adminURL')
        expected_admin_url = 'https://fake.api.globoi.com/v1/AUTH_1'

        public_url = utils.get_storage_endpoint(request, 'publicURL')
        expected_public_url = 'http://fake.s3.glbimg.com/v1/AUTH_1'

        internal_url = utils.get_storage_endpoint(request, 'internalURL')
        expected_internal_url = 'http://fake.i.s3.glbimg.com/v1/AUTH_1'

        self.assertEqual(admin_url, expected_admin_url)
        self.assertEqual(public_url, expected_public_url)
        self.assertEqual(internal_url, expected_internal_url)

    def test_get_storage_endpoint_not_found_return_none(self):
        user = fakes.FakeUser(1, 'user')
        request = fake_request(user=user)
        endpoint = utils.get_storage_endpoint(request, 'Invalid')
        self.assertEqual(endpoint, None)

    def test_get_storage_endpoint_without_service_catalog(self):
        user = fakes.FakeUser(1, 'user')
        request = fake_request(user=user)
        request.session['service_catalog'] = None

        endpoint = utils.get_storage_endpoint(request, 'internalURL')
        self.assertEqual(endpoint, None)



#!/usr/bin/env python

from getpass import getpass

from django.contrib.auth.models import User, Group
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('-s', '--superuser', action='store_true',
            help="Creates a superuser.", )
        parser.add_argument('-u', '--username', type=str,
            help="Username of the superuser.", )
        parser.add_argument('-e', '--email', type=str,
            help="E-mail of the superuser.", )
        parser.add_argument('-t', '--teamname', type=str,
            help="Team to which add the superuser.", )
        parser.add_argument('-p', '--password', type=str,
            help="Password of the superuser.", )

    def handle(self, *args, **kwargs):
        username = kwargs.get("username")

        if not username:
            username = input("Username: ")

        if not username:
            self.stdout.write(self.style.ERROR("Username cannot be blank."))
            exit()

        email = kwargs.get("email")

        if not email:
            email = input("E-mail: ")

        if not email:
            self.stdout.write(self.style.ERROR("E-mail cannot be blank."))
            exit()

        teamName = kwargs.get("teamname")

        if not teamName:
            teamName = input("Team: ")

        if not teamName:
            self.stdout.write(self.style.ERROR("Team cannot be blank."))
            exit()

        password = kwargs.get("password")
        if not password:
            password = getpass()
            if not password:
                self.stdout.write(self.style.ERROR(
                    "Password cannot be blank."))
                exit()
            if password != getpass(prompt='Confirm password: '):
                self.stdout.write(self.style.ERROR(
                    "Passwords do not match."))
                exit()

        try:
            grp = Group.objects.get(name=teamName)
            self.stdout.write(self.style.WARNING(
                "Group '{}' already exists!".format(teamName)))
        except Group.DoesNotExist:
            self.stdout.write("Creating group '{}'".format(teamName))
            grp = Group.objects.create(name=teamName)
            grp.save()
            self.stdout.write(self.style.SUCCESS(
                "Group '{}' successfully created!".format(teamName)))

        try:
            usr = User.objects.get(username=username)
            self.stdout.write(self.style.WARNING(
                "User '{}' already exists!".format(username)))
        except User.DoesNotExist:
            if kwargs.get("superuser"):
                self.stdout.write("Creating superuser '{}'".format(username))
                usr = User.objects.create_superuser(username, email, password)
                self.stdout.write(self.style.SUCCESS(
                    "Superuser '{}' successfully created!".format(username)))
            else:
                self.stdout.write("Creating user '{}'".format(username))
                usr = User.objects.create_user(username, email, password)
                self.stdout.write(self.style.SUCCESS(
                    "User '{}' successfully created!".format(username)))

        if usr.groups.filter(name=teamName).count() == 0:
            self.stdout.write("Adding user '{}' to group '{}'".format(
                username, teamName))
            usr.groups.add(grp)
            usr.save()
            self.stdout.write(self.style.SUCCESS(
                "User '{}' successfully added to group '{}'!".format(
                    username, teamName)))
        else:
            self.stdout.write(self.style.WARNING(
                "User '{}' already in group '{}'!".format(
                    username, teamName)))



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib.auth.models import Group, User
from django.utils.translation import gettext_lazy as _
from django.db.models.signals import class_prepared


class GroupProjects(models.Model):

    group = models.ForeignKey(Group, on_delete=models.CASCADE)
    project = models.CharField(max_length=255)
    owner = models.BooleanField(default=0)

    class Meta:
        db_table = 'group_projects'
        unique_together = (('group', 'project'),)
        verbose_name_plural = _('Groups and Projects')

    def __str__(self):
        return "Group {} - Project {}".format(self.group, self.project)


class CurrentProject(models.Model):

    user = models.OneToOneField(User, on_delete=models.CASCADE)
    project = models.CharField(max_length=255)

    class Meta:
        db_table = 'current_project'
        verbose_name_plural = _('Current Project')



# -*- coding: utf-8 -*-

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class StorageConfig(AppConfig):
    name = 'storage'
    verbose_name = _("Swift Browser")
    vault_app = True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-12-03 22:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BackupContainer',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('container', models.CharField(max_length=255)),
                ('project_id', models.CharField(max_length=255)),
                ('project_name', models.CharField(max_length=255)),
                ('created_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'swift_backup_container',
            },
        ),
        migrations.AlterUniqueTogether(
            name='backupcontainer',
            unique_together=set([('container', 'project_id')]),
        ),
    ]



import logging

from django.conf import settings
from django.shortcuts import render
from django.contrib.auth.decorators import login_required

from swiftclient import client

from storage.utils import get_storage_endpoint
from vault.utils import update_default_context

log = logging.getLogger(__name__)


@login_required
def accountview(request, project):
    storage_url = get_storage_endpoint(request, 'adminURL')

    if not storage_url:
        return

    auth_token = request.session.get('auth_token')
    http_conn = client.http_connection(
        storage_url, insecure=settings.SWIFT_INSECURE)

    head_acc = {}
    try:
        head_acc = client.head_account(
            storage_url, auth_token, http_conn=http_conn)
    except Exception as err:
        log.exception(f'Exception: {err}')

    context = {
        'cloud': head_acc.get('x-account-meta-cloud'),
        'total_containers': head_acc.get('x-account-container-count'),
        'total_objects': head_acc.get('x-account-object-count'),
        'total_bytes': head_acc.get('x-account-bytes-used'),
        'account_domain_id': head_acc.get('x-account-project-domain-id'),
    }

    return render(request, 'storage/accountview.html',
                  update_default_context(request, context))



# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings
from django.core.validators import RegexValidator
from django.forms.fields import ChoiceField

from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import Group

from vault.widgets import PasswordInputWithEye
from identity.keystone import Keystone

BOOLEAN_CHOICES = (
    (True, 'Yes'),
    (False, 'No')
)
CLOUD_CHOICES = (
    (None, 'No Cloud'),
    ('gcp', 'Google Cloud')
)


class UserForm(forms.Form):

    id = forms.CharField(widget=forms.HiddenInput(), required=False)
    id = forms.CharField(widget=forms.HiddenInput(), required=False)

    name = forms.CharField(
        label=_('User Name'),
        required=True,
        widget=forms.TextInput(attrs={'class': 'form-control'}))

    email = forms.EmailField(
        label=_('Email'),
        required=False,
        widget=forms.TextInput(attrs={'class': 'form-control'}))

    password = forms.CharField(
        label=_('Password'),
        required=True,
        widget=PasswordInputWithEye(attrs={'class': 'form-control'},
                                   render_value=True))

    password_confirm = forms.CharField(
        label=_('Confirm Password'),
        required=True,
        widget=PasswordInputWithEye(attrs={'class': 'form-control'},
                                   render_value=True))

    project = ChoiceField(
        label=_('Primary Project'),
        required=True,
        widget=forms.Select(attrs={'class': 'form-control'}))

    enabled = forms.BooleanField(
        label=_('Enabled'),
        required=False,
        widget=forms.Select(attrs={'class': 'form-control'},
                            choices=BOOLEAN_CHOICES), initial=True)

    def clean_password(self):
        if 'password' in self.data:
            if self.data['password'] != self.data.get('password_confirm'):
                raise forms.ValidationError(_('Passwords did not match'))

            return self.data['password']


class CreateUserForm(UserForm):

    role = ChoiceField(label=u'Role', required=True,
                       widget=forms.Select(attrs={'class': 'form-control'}))

    def __init__(self, *args, **kwargs):
        super(CreateUserForm, self).__init__(*args, **kwargs)

        _password = Keystone.create_password()

        for field in ('password', 'password_confirm'):
            self.fields[field].initial = _password


class UpdateUserForm(UserForm):

    def __init__(self, *args, **kwargs):
        super(UpdateUserForm, self).__init__(*args, **kwargs)

        for field in ('password', 'password_confirm', 'project'):
            self.fields[field].required = False

        self.fields['project'].widget.attrs['disabled'] = 'True'


class ProjectForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(ProjectForm, self).__init__(*args, **kwargs)

        user = kwargs.get('initial').get('user')

        instance = getattr(self, 'instance', None)
        if instance and instance.id:
            self.fields['group'].required = True

        action = kwargs.get('initial').get('action')

        if action == 'update':
            self.fields['name'].widget.attrs['readonly'] = 'True'

    id = forms.CharField(widget=forms.HiddenInput(), required=False)

    action = forms.CharField(widget=forms.HiddenInput(), required=False)

    name = forms.CharField(
        label=_('Project Name'),
        required=True,
        widget=forms.TextInput(attrs={'class': 'form-control'}),
        validators=[
            RegexValidator(
                '^[a-zA-Z0-9_-]*$',
                message=_('Project Name must be an alphanumeric.')
            )
        ]
    )

    description = forms.CharField(
        label=_('Description'),
        required=True,
        widget=forms.Textarea(attrs={'class': 'form-control', 'rows': 5}))

    enabled = forms.BooleanField(
        label=_('Enabled'),
        required=False,
        widget=forms.Select(
            attrs={'class': 'form-control'},
            choices=BOOLEAN_CHOICES),
        initial=True)

    group = forms.ModelChoiceField(
        label=_('Group'),
        required=True,
        widget=forms.Select(attrs={'class': 'form-control'}),
        queryset=Group.objects.all())

    if settings.SWIFT_CLOUD_ENABLED:
        cloud = forms.ChoiceField(
            label=_('Cloud Provider'),
            required=False,
            choices=CLOUD_CHOICES,
            widget=forms.Select(attrs={'class': 'form-control'}),
            initial=True)

    def clean_description(self):
        if 'description' in self.data:
            description = self.data['description']
            if len(description.strip()) == 0:
                raise forms.ValidationError(
                    _('Project description cannot be empty.'))

            return self.data['description']


class DeleteProjectConfirm(forms.Form):

    user = forms.CharField(
        label=_('User'),
        required=True,
        widget=forms.TextInput(
            attrs={
                'class': 'form-control',
                'placeholder': _('Confirm project user'),
            }
        )
    )

    password = forms.CharField(
        label=_('Password'),
        required=True,
        widget=PasswordInputWithEye(
            attrs={
                'class': 'form-control',
                'placeholder': _('Confirm password')
            }
        )
    )



# -*- coding: utf-8 -*-

import logging

from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import Group
from django.utils.translation import gettext_lazy as _

from keystoneclient import exceptions

from vault.models import GroupProjects
from vault.forms import GroupAdminForm, GroupProjectsForm
from identity.keystone import KeystoneNoRequest


log = logging.getLogger(__name__)


def get_project_list():
    project_list = {}
    try:
        keystone = KeystoneNoRequest()
        if keystone.conn is not None:
            for project in keystone.project_list():
                project_list[project.id] = project.name
    except exceptions.AuthorizationFailure:
        msg = _('Unable to retrieve Keystone data')
        # messages.add_message(request, messages.ERROR, msg)
        log.error(f'In get_project_list(): {msg}')
    except Exception as e:
        pass

    return project_list


class GroupProjectsAdmin(admin.ModelAdmin):

    def __init__(self, *args, **kwargs):
        self.project_list = get_project_list()
        super(GroupProjectsAdmin, self).__init__(*args, **kwargs)

    def __call__(self, *args, **kwargs):
        self.project_list = get_project_list()
        super(GroupProjectsAdmin, self).__call__(*args, **kwargs)

    form = GroupProjectsForm
    list_display = ('group_name', 'project_name')

    def group_name(self, obj):
        return obj.group.name
    group_name.allow_tags = True
    group_name.short_description = 'group'

    def project_name(self, obj):
        return self.project_list.get(obj.project, '')
    project_name.allow_tags = True
    project_name.short_description = 'project'


class GroupAdmin(admin.ModelAdmin):
    form = GroupAdminForm
    filter_horizontal = ['permissions']
    search_fields = ['name']


admin.site.register(GroupProjects, GroupProjectsAdmin)
admin.site.unregister(Group)
admin.site.register(Group, GroupAdmin)
admin.site.site_header = "Vault Admin"



import json

from urllib.parse import urlencode
from django import template
from django.conf import settings
from django.apps import apps

from vault.models import GroupProjects
from identity.keystone import KeystoneNoRequest

register = template.Library()


@register.simple_tag(takes_context=True)
def get_vault_env(context):
    request = context.get("request")
    envs = ["dev", "qa", "qa1", "qa2", "prod", "beta", "docker"]

    if settings.ENVIRON in envs:
        return settings.ENVIRON

    if settings.ENVIRON is None and "localhost" in request.get_host():
        return "local"

    return ""


@register.inclusion_tag('vault/set_project.html', takes_context=True)
def set_project(context):
    user = context.get('user')
    groups = user.groups.all()
    request = context.get('request')
    keystone = KeystoneNoRequest()
    group_projects = []

    for group in groups:
        gps = GroupProjects.objects.filter(group=group.id)
        gps_ks = [x for x in
                filter(lambda x: x.enabled, keystone.project_list())]
        gp_projs = []

        for gp in gps:
            for gp_ks in gps_ks:
                if gp.project == gp_ks.id:
                    gp_projs.append(gp_ks)
                    break

        gp_projs.sort(key=lambda x: x.name.lower())
        group_projects.append({
            'team': group.name,
            'projects': gp_projs
        })

    current_project = {'id': context.get('project_id'),
                       'name': context.get('project_name')}

    if current_project.get('id') is None:
        req = context.get('request')
        current_project['id'] = req.session.get('project_id')
        current_project['name'] = req.session.get('project_name')

    return {
        'current_project': current_project,
        'group_projects': group_projects,
        'has_group': user.groups.count() > 0
    }


@register.simple_tag(takes_context=True)
def get_logout_url(context):
    request = context.get('request')
    logout_url = settings.LOGOUT_URL.format(
        request.META['HTTP_HOST']
    )
    return logout_url


@register.simple_tag()
def url_replace(get_parameters, **kwargs):
    query = get_parameters
    query.update(kwargs)
    for x in query:
        if type(query[x]) is not list:
            query[x] = [query[x]]
    query_pairs = [(k, v) for k, vlist in query.items() for v in vlist]
    return urlencode(query_pairs)



# -*- coding: utf-8 -*-

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ActionloggerConfig(AppConfig):
    name = 'actionlogger'
    verbose_name = _("Actionlogger")



# -*- coding: utf-8 -*-

"""
Unit tests for utils functions
"""

import datetime

from unittest import TestCase
from unittest.mock import patch

from keystoneclient.v3.projects import Project

from vault import utils
from vault.models import CurrentProject
from django.core.paginator import PageNotAnInteger, EmptyPage
from vault.tests.fakes import fake_request, UserFactory


class TestVaultUtils(TestCase):

    def setUp(self):
        # does not connect to the keystone client
        patch('keystoneclient.v3.client.Client').start()

    def tearDown(self):
        patch.stopall()

    def test_generic_pagination(self):
        items = [1, 2, 3, 4, 5, 6]

        # First page
        paginated_items = utils.generic_pagination(items, 1, 2)
        computed = [k for k in paginated_items]
        expected = [1, 2]
        self.assertEqual(computed, expected)
        self.assertFalse(paginated_items.has_previous())
        self.assertTrue(paginated_items.has_next())

        # Second page
        paginated_items = utils.generic_pagination(items, 2, 2)
        computed = [k for k in paginated_items]
        expected = [3, 4]
        self.assertEqual(computed, expected)
        self.assertTrue(paginated_items.has_previous())
        self.assertTrue(paginated_items.has_next())

        # Last page
        paginated_items = utils.generic_pagination(items, 3, 2)
        computed = [k for k in paginated_items]
        expected = [5, 6]
        self.assertEqual(computed, expected)
        self.assertTrue(paginated_items.has_previous())
        self.assertFalse(paginated_items.has_next())

        # If page bigger than the last page, always return last page
        paginated_items = utils.generic_pagination(items, 4, 2)
        computed = [k for k in paginated_items]
        expected = [5, 6]
        self.assertEqual(computed, expected)

    @patch('django.core.paginator.Paginator')
    def test_generic_pagination_return_except_emptypage(self, mock_paginator):
        mock_paginator.side_effect = EmptyPage()

        items = [1, 2, 3, 4, 5, 6]

        paginated_items = utils.generic_pagination(items, 1, 2)

        self.assertEqual(paginated_items.next_page_number(), 2)

    @patch('django.core.paginator.Paginator')
    def test_generic_pagination_return_except_pagenotinteger(self, mock_paginator):
        mock_paginator.side_effect = PageNotAnInteger()

        items = [1, 2, 3, 4, 5, 6]

        paginated_items = utils.generic_pagination(items, 1, 2)

        self.assertEqual(paginated_items.number, 1)

    @patch('identity.keystone.KeystoneNoRequest.project_get_by_name')
    def test_set_current_project(self, mock_project):
        project = Project('123', {})
        project.id = 1
        project.name = 'Teste'

        mock_project.return_value = project

        request = fake_request(method='GET')
        request.user = UserFactory(id='999', username='u_user_test')
        patch('identity.keystone.Keystone._create_keystone_connection').start()

        project_name = request.session['project_name']
        utils.set_current_project(request, project)

        self.assertEqual(request.session['project_id'], 1)
        self.assertEqual(request.session['project_name'], 'Teste')

    @patch('vault.models.CurrentProject.objects.get')
    @patch('identity.keystone.KeystoneNoRequest.project_get')
    def test_get_current_project(self, mock_project, mock_current_project):
        mock_current_project.return_value = CurrentProject()

        project = Project('123', {})
        project.id = 1
        project.name = 'Nome'

        mock_project.return_value = project

        result_project = utils.get_current_project('1')

        self.assertEqual(result_project.id, project.id)
        self.assertEqual(result_project.name, project.name)

    @patch('vault.models.CurrentProject.objects.get')
    def test_get_current_project_return_none(self, mock_current_project):
        mock_current_project.side_effect = CurrentProject.DoesNotExist()

        result_project = utils.get_current_project('1')

        self.assertEqual(result_project, None)

    @patch('identity.keystone.Keystone._create_keystone_connection')
    def test_maybe_update_token_without_time_token_value(self, keystone_conn_mock):
        conn_mock = keystone_conn_mock.return_value
        conn_mock.auth_token = '12345678'

        request = fake_request(method='GET')
        request.session['token_time'] = None
        result = utils.maybe_update_token(request)

        self.assertTrue(result)

    @patch('identity.keystone.Keystone._create_keystone_connection')
    def test_maybe_update_token_with_expired_time_token(self, keystone_conn_mock):
        conn_mock = keystone_conn_mock.return_value
        conn_mock.auth_token = '12345678'

        request = fake_request(method='GET')
        expired_time = datetime.datetime.utcnow() - datetime.timedelta(minutes=16)
        request.session['token_time'] = expired_time
        result = utils.maybe_update_token(request)

        self.assertTrue(result)

    @patch('identity.keystone.Keystone._create_keystone_connection')
    def test_maybe_update_token_with_valid_time_token(self, keystone_conn_mock):
        conn_mock = keystone_conn_mock.return_value
        conn_mock.auth_token = '12345678'

        request = fake_request(method='GET')
        valid_time = datetime.datetime.utcnow() + datetime.timedelta(minutes=15)
        request.session['token_time'] = valid_time
        result = utils.maybe_update_token(request)

        self.assertFalse(result)



# -*- coding: utf-8 -*-

import random
import factory
from datetime import datetime, timedelta

from io import StringIO

from django.core.handlers.wsgi import WSGIRequest
from django.contrib.auth.models import User, Group, AnonymousUser
from django.contrib.messages.storage import default_storage
from django.contrib.sessions.backends.db import SessionStore


class GroupFactory(factory.django.DjangoModelFactory):

    class Meta:
        model = Group
        django_get_or_create = ('name',)

    name = factory.Sequence(lambda n: "Group #{}".format(n))


class UserFactory(factory.django.DjangoModelFactory):

    class Meta:
        model = User
        django_get_or_create = ('username',)

    username = factory.Sequence(lambda n: "User #{}".format(n))
    is_superuser = True

    @factory.post_generation
    def groups(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            for group in extracted:
                self.groups.add(group)


def fake_request(path='/', method='GET', user=True, extra={}):
    params = {
        'REQUEST_METHOD': method,
        'PATH_INFO': path,
        'wsgi.input': StringIO()
    }
    params.update(extra)

    req = WSGIRequest(params)

    if user:
        req.user = UserFactory.create(groups=(GroupFactory(), GroupFactory()))
    else:
        req.user = AnonymousUser()

    req.build_absolute_uri = lambda x=None: '/'

    # for sessions middleware
    req.session = build_fake_session()

    # for messages middleware
    req._messages = default_storage(req)

    req.get_host = lambda x=None: 'localhost'

    return req


def build_fake_session():
    fake_session = SessionStore()
    fake_session['token_time'] = timedelta(minutes=15) + datetime.utcnow()
    fake_session['project_id'] = '1'
    fake_session['project_name'] = 'fake_project'
    fake_session['service_catalog'] = {
        'object_store': {'adminURL': 'https://fake.api.globoi.com/v1/AUTH_XPTO',
                         'publicURL': 'http://fake.s3.glbimg.com/v1/AUTH_XPTO',
                         'internalURL': 'http://fake.i.s3.glbimg.com/v1/AUTH_XPTO'},

        'identity': {'adminURL': 'http://identity/admin',
                     'publicURL': 'http://identity/public',
                     'internalURL': 'http://identity/internal'},

        'block_storage': {'adminURL': 'http://block_storage/XPTO',
                          'publicURL': 'http://block_storage/XPTO',
                          'internalURL': 'http://block_storage/XPTO'}
    }

    return fake_session



# -*- coding: utf-8 -*-

import logging

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import Group
from django.urls import reverse, reverse_lazy
from django.http import HttpResponseRedirect
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.debug import sensitive_post_parameters
from django.views.generic.base import View, TemplateView
from django.views.generic.edit import FormView

from keystoneclient import exceptions

from storage.utils import delete_swift_account, update_swift_account
from actionlogger.actionlogger import ActionLogger
from identity.keystone import Keystone
from identity.forms import UserForm, CreateUserForm, UpdateUserForm, ProjectForm, DeleteProjectConfirm

from vault.jsoninfo import JsonInfo
from vault import utils
from vault.models import GroupProjects
from vault.views import SuperUserMixin, JSONResponseMixin, LoginRequiredMixin, ProjectCheckMixin


log = logging.getLogger(__name__)
actionlog = ActionLogger()


class WithKeystoneMixin:
    def __init__(self, *args, **kwargs):
        self.keystone = None
        super(WithKeystoneMixin, self).__init__(*args, **kwargs)

    def dispatch(self, request, *args, **kwargs):
        try:
            self.keystone = Keystone(request)
        except exceptions.AuthorizationFailure:
            msg = _("Unable to retrieve Keystone data")
            messages.add_message(request, messages.ERROR, msg)
            log.error(f"{request.user}: {msg}")

            # Redirects user back to previous page
            return HttpResponseRedirect(request.META.get("HTTP_REFERER", "/"))

        if self.keystone.conn is None:
            msg = _("Authorization error")
            messages.add_message(request, messages.ERROR, msg)
            log.error("Keystone: {}".format(msg))

        return super(WithKeystoneMixin, self).dispatch(request, *args, **kwargs)


class ListUserView(SuperUserMixin, WithKeystoneMixin, ProjectCheckMixin, TemplateView):
    template_name = "identity/users.html"

    def get_context_data(self, **kwargs):
        context = super(ListUserView, self).get_context_data(**kwargs)
        page = self.request.GET.get("page", 1)

        users = []

        try:
            users = self.keystone.user_list()
        except Exception as e:
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
            messages.add_message(self.request, messages.ERROR, _("Unable to list users"))

        sorted_users = sorted(users, key=lambda l: l.name.lower())
        context["users"] = utils.generic_pagination(sorted_users, page)

        return context


class BaseUserView(SuperUserMixin, WithKeystoneMixin, FormView):
    form_class = UserForm

    def _fill_project_choices(self, form):
        if self.keystone and "project" in form.fields:
            items = [("", "---")]

            for i in self.keystone.project_list():
                if getattr(i, "enabled", None):
                    items.append((i.id, i.name))

            form.fields["project"].choices = items

    def _fill_role_choices(self, form):
        if self.keystone and "role" in form.fields:
            items = [("", "---")]

            for i in self.keystone.role_list():
                items.append((i.id, i.name))

            index = [items.index(item) for item in items if "_member_" in item]

            if index:
                items.pop(index[0])

            form.fields["role"].choices = items

    @method_decorator(sensitive_post_parameters("password", "password_confirm"))
    def dispatch(self, request, *args, **kwargs):
        project_name = request.session.get("project_name")
        self.success_url = reverse_lazy("admin_list_users", kwargs={"project": project_name})
        return super(BaseUserView, self).dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        form = self.get_form(self.form_class)

        self._fill_project_choices(form)
        self._fill_role_choices(form)

        return self.render_to_response(self.get_context_data(form=form, **kwargs))

    def get_context_data(self, **kwargs):
        context = super(BaseUserView, self).get_context_data(**kwargs)
        user_id = kwargs.get("user_id")
        form = kwargs.get("form")

        if not user_id:
            user_id = form.data.get("id")

        if user_id:
            user = self.keystone.user_get(user_id)
            form.initial = user.to_dict()
            form.fields["project"].initial = user.default_project_id
            context["user_id"] = user_id

        return context


class CreateUserView(BaseUserView):
    form_class = CreateUserForm
    template_name = "identity/user_create.html"

    def post(self, request, *args, **kwargs):
        form = self.get_form(self.form_class)

        self._fill_project_choices(form)
        self._fill_role_choices(form)

        if form.is_valid():
            post = request.POST

            enabled = False if post.get("enabled") in ("False", "0") else True

            try:
                user = self.keystone.user_create(
                    name=post.get("name"),
                    email=post.get("email"),
                    password=post.get("password"),
                    project_id=post.get("project"),
                    enabled=enabled,
                    domain=post.get("domain"),
                    role_id=post.get("role"),
                )

                messages.add_message(request, messages.SUCCESS, _("Successfully created user"))
                actionlog.log(request.user.username, "create", user)

            except Exception as e:
                log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
                messages.add_message(request, messages.ERROR, _("Error when creating user"))

            return self.form_valid(form)
        else:
            return self.form_invalid(form)


class UpdateUserView(BaseUserView):
    form_class = UpdateUserForm
    template_name = "identity/user_edit.html"

    def post(self, request, *args, **kwargs):
        form = self.get_form(self.form_class)

        self._fill_project_choices(form)

        if form.is_valid():
            post = request.POST
            enabled = False if post.get("enabled") in ("False", "0") else True

            try:
                user = self.keystone.user_get(post.get("id"))

                # can't modify primary project
                project = self.keystone.project_get(user.default_project_id)

                self.keystone.user_update(
                    user,
                    name=post.get("name"),
                    email=post.get("email"),
                    password=post.get("password"),
                    project=project.id,
                    enabled=enabled,
                    domain=post.get("domain"),
                )

                messages.add_message(request, messages.SUCCESS, _("Successfully updated user"))
                actionlog.log(request.user.username, "update", user)

            except Exception as e:
                log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
                messages.add_message(request, messages.ERROR, _("Error when updating user"))

        context = self.get_context_data(form=form, request=request)
        return self.render_to_response(context)


class DeleteUserView(BaseUserView):
    def get(self, request, *args, **kwargs):
        try:
            self.keystone.user_delete(kwargs.get("user_id"))
            messages.add_message(request, messages.SUCCESS, _("Successfully deleted user"))
            actionlog.log(request.user.username, "delete", "user_id: {}".format(kwargs.get("user_id")))

        except Exception as e:
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
            messages.add_message(request, messages.ERROR, _("Error when deleting user"))

        project_name = request.session.get("project_name")
        success_url = reverse("admin_list_users", kwargs={"project": project_name})

        return HttpResponseRedirect(success_url)


class BaseProjectView(LoginRequiredMixin, WithKeystoneMixin, FormView):
    success_url = reverse_lazy("change_project")

    def get(self, request, *args, **kwargs):

        # edit_project is the non admin project update view
        if request.resolver_match is not None and request.resolver_match.url_name == "edit_project":

            project = self.keystone.project_get(kwargs.get("project_id"))

            form = ProjectForm(initial={"user": request.user, "action": "update"})
        else:
            form = ProjectForm(initial={"user": request.user})

        # Show only user groups
        form.fields["group"].queryset = request.user.groups

        context = self.get_context_data(form=form, request=request, **kwargs)

        return self.render_to_response(context)

    def get_context_data(self, **kwargs):

        request = kwargs.get("request")
        context = super(BaseProjectView, self).get_context_data(**kwargs)
        form = kwargs.get("form")

        project_id = kwargs.get("project_id")
        if not project_id:
            project_id = form.data.get("id")

        # Show role manager when superuser
        context["show_roles"] = request.user.is_superuser

        if project_id:
            project = self.keystone.project_get(project_id).to_dict()
            form.initial = project
            context["project"] = project
            context["identity_project_id"] = project_id
            context["has_id"] = True

            try:
                form.initial["group"] = GroupProjects.objects.get(project=project_id, owner=1).group_id
            except GroupProjects.DoesNotExist:
                form.initial["group"] = None

            if project.get("team_owner_id") is not None:
                try:
                    context["project"]["team"] = Group.objects.get(id=project["team_owner_id"])
                except Group.DoesNotExist:
                    context["project"]["team"] = None

            if project.get("first_team_id") is not None:
                try:
                    context["project"]["first_team"] = Group.objects.get(id=project["first_team_id"])
                except Group.DoesNotExist:
                    context["project"]["first_team"] = None

            user = self.keystone.find_user_with_u_prefix(project_id, "u")
            if user:
                context["user_project"] = user.name

            if context["show_roles"]:
                try:
                    users = self.keystone.user_list()
                    context["users"] = sorted(users, key=lambda l: l.name.lower())

                    roles = self.keystone.role_list()
                    context["roles"] = sorted(roles, key=lambda l: l.name.lower())
                except Exception as e:
                    log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))

        return context


class ListProjectView(SuperUserMixin, WithKeystoneMixin, ProjectCheckMixin, TemplateView):
    template_name = "identity/projects.html"

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)

        return self.render_to_response(context)

    def get_context_data(self, **kwargs):
        context = super(ListProjectView, self).get_context_data(**kwargs)
        page = self.request.GET.get("page", 1)

        context["projects"] = utils.generic_pagination(self._get_data(), page)

        return context

    def _get_data(self):
        """Retrieve sorted list of projects"""
        projects = []
        try:
            projects = [p.to_dict() for p in self.keystone.project_list()]
            projects = sorted(projects, key=lambda l: l["name"].lower())

            for prj in projects:
                prj["team"] = ""
                if prj.get("team_owner_id") is not None:
                    prj["team"] = Group.objects.filter(id=prj["team_owner_id"]).first()
        except Exception as e:
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
            messages.add_message(self.request, messages.ERROR, _("Unable to list projects"))

        return projects


class CreateProjectSuccessView(LoginRequiredMixin, TemplateView):
    template_name = "identity/project_create_success.html"

    @method_decorator(utils.project_required)
    def get(self, request, *args, **kwargs):
        context = self.get_context_data(request=request, **kwargs)
        return self.render_to_response(context)

    def get_context_data(self, **kwargs):
        context = super(CreateProjectSuccessView, self).get_context_data(**kwargs)

        request = kwargs.get("request")

        context["project_info"] = request.session.get("project_info")
        context["project_info"]["auth_url"] = settings.KEYSTONE_URL

        project_name = context["project_info"]["project_name"]
        user_name = context["project_info"]["user_name"]
        password = context["project_info"]["user_password"]

        try:
            keystone = Keystone(request, username=user_name, password=password, project_name=project_name)
            context["project_info"]["endpoints"] = keystone.get_endpoints()
        except exceptions.AuthorizationFailure:
            msg = _("Unable to retrieve Keystone data")
            messages.add_message(request, messages.ERROR, msg)
            log.error(f"{request.user}: {msg}")

        return context


class CreateProjectView(BaseProjectView):
    template_name = "identity/project_create.html"
    form_class = ProjectForm
    success_url = reverse_lazy("change_project")

    def post(self, request, *args, **kwargs):
        form = ProjectForm(initial={"user": request.user}, data=request.POST)

        if not form.is_valid():
            return self.render_to_response(
                self.get_context_data(form=form, request=request))

        group_id = request.POST.get("group")
        response = self.keystone.vault_project_create(
            request.POST.get("name"),
            group_id,
            description=request.POST.get("description"),
            first_team_id=group_id,
            team_owner_id=group_id,
            created_by="vault",
        )

        # Houve falha no cadastro
        if not response.get("status"):
            log.exception("Exception: {}".format(response.get("status")))
            messages.add_message(request, messages.ERROR, response.get("reason"))

            return self.render_to_response(
                self.get_context_data(form=form, request=request))

        project = response.get("project")
        user = response.get("user")
        password = response.get("password")

        # swift-cloud middleware
        if settings.SWIFT_CLOUD_ENABLED:
            cloud = request.POST.get("cloud")
            if cloud != "":
                update_swift_account(user.name, password,
                    project.name, {"x-account-meta-cloud": cloud})

        actionlog.log(request.user.username, "create", project)
        actionlog.log(request.user.username, "create", user)

        request.session["project_info"] = {
            "user_name": user.name,
            "project_name": project.name,
            "user_password": password,
        }

        return redirect("create_project_success", project=project.name)


class ChangeProjectView(LoginRequiredMixin, WithKeystoneMixin, TemplateView):
    template_name = "identity/project_change.html"

    def get(self, request, *args, **kwargs):

        if self.request.user.groups.count() == 0:
            self.template_name = "identity/project_change_noteam.html"

        context = self.get_context_data(request=request, **kwargs)

        return self.render_to_response(context)

    def get_context_data(self, **kwargs):
        context = super(ChangeProjectView, self).get_context_data(**kwargs)
        page = self.request.GET.get("page", 1)

        context["projects"] = utils.generic_pagination(self._get_data(), page)

        return context

    def _get_data(self):
        """Retrieve sorted list of projects"""
        user = self.request.user
        groups = user.groups.all()
        try:
            keystone = Keystone(self.request)
        except exceptions.AuthorizationFailure:
            msg = _("Unable to retrieve Keystone data")
            messages.add_message(self.request, messages.ERROR, msg)
            log.error(f"{self.request.user}: {msg}")

            return []

        projects = []

        for group in groups:
            gps = GroupProjects.objects.filter(group=group.id)
            gps_ks = [x for x in filter(lambda x: x.enabled, keystone.project_list())]

            for gp in gps:
                for gp_ks in gps_ks:
                    if gp.project == gp_ks.id and gp_ks not in projects:
                        if self.request.GET.get('project_name', '') in gp_ks.name:
                            projects.append(gp_ks)
                        break

            projects.sort(key=lambda x: x.name.lower())

        return projects


class UpdateProjectView(BaseProjectView):
    template_name = "identity/project_edit.html"

    def post(self, request, *args, **kwargs):
        post = request.POST
        project = self.keystone.project_get(post.get("id"))

        form = ProjectForm(data=post, initial={"user": request.user})

        if form.is_valid():
            enabled = True
            if post.get("enabled") in ["False", "0"]:
                enabled = False

            group_id = post.get("group")

            description = post.get("description")
            if description == "":
                description = None

            try:
                response = self.keystone.vault_project_update(
                    project.id, project.name, group_id, description=description, enabled=enabled, team_owner_id=group_id
                )

                if response["status"]:
                    messages.add_message(request, messages.SUCCESS, _("Successfully updated project"))
                else:
                    messages.add_message(request, messages.ERROR, _(response["reason"]))

                actionlog.log(request.user.username, "update", project)
            except Exception as e:
                log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
                messages.add_message(request, messages.ERROR, _("Error when updating project"))

        context = self.get_context_data(form=form, request=request)
        return self.render_to_response(context)


class DeleteProjectView(BaseProjectView):
    template_name = "identity/project_delete_confirm.html"

    def get(self, request, *args, **kwargs):
        form = DeleteProjectConfirm()
        return self.render_to_response(self.get_context_data(form=form, request=request))

    def post(self, request, *args, **kwargs):
        form = DeleteProjectConfirm(data=request.POST)

        if not form.is_valid():
            return self.render_to_response(self.get_context_data(form=form, request=request))

        user = form.data.get("user")
        password = form.data.get("password")

        project_id = self.kwargs.get("project_id")
        project_name = self.keystone.project_get(project_id).name

        try:
            keystone_app = Keystone(request, username=user, password=password, project_name=project_name)
        except exceptions.Unauthorized:
            # Fail to authenticate with sent credentials
            messages.add_message(request, messages.ERROR, _("Invalid credentials."))

            return self.render_to_response(context=self.get_context_data(form=form, request=request))
        except exceptions.AuthorizationFailure:
            msg = _("Unable to retrieve Keystone data")
            messages.add_message(request, messages.ERROR, msg)
            log.error(f"{request.user}: {msg}")

            return self.render_to_response(context=self.get_context_data(form=form, request=request))

        endpoints = keystone_app.get_endpoints()
        storage_url = endpoints.get("object_store").get("adminURL")
        auth_token = self.keystone.conn.auth_token

        swift_del_result = delete_swift_account(storage_url, auth_token)

        if not swift_del_result:
            messages.add_message(request, messages.ERROR, _("Error when deleting swift account"))

            return HttpResponseRedirect(
                reverse("edit_project", kwargs={"project_id": project_id, "project": project_name})
            )

        try:
            self.keystone.vault_project_delete(project_name)
            messages.add_message(request, messages.SUCCESS, _("Successfully deleted project."))

        except Exception as e:
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
            messages.add_message(request, messages.ERROR, _("Error when deleting project"))

        # Purge project from current projects
        utils.purge_current_project(request, project_id)

        project_name = request.session.get("project_name")
        success_url = reverse("change_project")

        return HttpResponseRedirect(success_url)


class ListUserRoleView(SuperUserMixin, WithKeystoneMixin, View, JSONResponseMixin):
    def post(self, request, *args, **kwargs):
        project_id = request.POST.get("project")
        context = {}

        if not project_id:
            return self.render_to_response(context)

        try:
            project_users = self.keystone.user_list(project_id=project_id)

            context["users"] = []
            unique_users = set()

            for user in project_users:
                if user.name not in unique_users:
                    unique_users.add(user.name)
                    context["users"].append(
                        {"id": user.id, "username": user.name, "roles": self.get_user_roles(user, project_id)}
                    )

            # sorting users by username
            context["users"] = [x for x in sorted(context["users"], key=lambda x: x.get("username"))]

            return self.render_to_response(context)

        except Exception as e:
            context["msg"] = "Error listing users"
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))

            return self.render_to_response(context, status=500)

    def get_user_roles(self, user, project_id):
        return [{"id": r.id, "name": r.name} for r in self.keystone.list_user_roles(user, project_id)]


class AddUserRoleView(SuperUserMixin, WithKeystoneMixin, View, JSONResponseMixin):
    def post(self, request, *args, **kwargs):
        project = request.POST.get("project")
        role = request.POST.get("role")
        user = request.POST.get("user")

        context = {"msg": "ok"}

        try:
            self.keystone.add_user_role(project=project, role=role, user=user)

            item = "project: {}, role: {}, user: {}".format(project, role, user)
            actionlog.log(request.user.username, "create", item)

            return self.render_to_response(context)

        except exceptions.Conflict as e:
            context["msg"] = str(_("User already registered with this role"))
            log.exception("{}{}".format(_("Conflict:"), e))
            return self.render_to_response(context, status=500)

        except Exception as e:
            context["msg"] = str(_("Error adding user"))
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
            return self.render_to_response(context, status=500)


class DeleteUserRoleView(SuperUserMixin, WithKeystoneMixin, View, JSONResponseMixin):
    def post(self, request, *args, **kwargs):
        project = request.POST.get("project")
        role = request.POST.get("role")
        user = request.POST.get("user")

        context = {"msg": "ok"}

        try:
            self.keystone.remove_user_role(project=project, role=role, user=user)

            item = "project: {}, role: {}, user: {}".format(project, role, user)
            actionlog.log(request.user.username, "delete", item)

            return self.render_to_response(context)

        except Exception as e:
            context["msg"] = str(_("Error removing user"))
            log.exception("Exception: {}".format(e))
            return self.render_to_response(context, status=500)


class UpdateProjectUserPasswordView(LoginRequiredMixin, WithKeystoneMixin, View, JSONResponseMixin):
    def post(self, request, *args, **kwargs):
        project_id = request.POST.get("project")
        context, status = {}, 200

        try:
            user = self.keystone.find_user_with_u_prefix(project_id, "u")
            new_password = Keystone.create_password()

            self.keystone.user_update(user, password=new_password)
            context = {"new_password": new_password}
            actionlog.log(request.user.username, "update", user)

        except Exception as e:
            context = {"msg": str(_("Error updating password"))}
            log.exception("Exception: {}".format(e))
            status = 500

        return self.render_to_response(context, status=status)


class KeystoneJsonInfo(JsonInfo, WithKeystoneMixin):
    def __init__(self, *args, **kwargs):
        self.keystone = kwargs["keystone"]
        super(KeystoneJsonInfo, self).__init__(*args, **kwargs)

    def generate_menu_info(self):
        project_name = self.request.session.get("project_name")
        return {
            "name": str(_("Identity")),
            "icon": "fas fa-key",
            "url": reverse("projects", kwargs={"project": project_name}),
            "subitems": [
                {"name": str(_("Projects")), "icon": "", "url": reverse("projects", kwargs={"project": project_name})},
                {
                    "name": str(_("Users")),
                    "icon": "",
                    "url": reverse("admin_list_users", kwargs={"project": project_name}),
                },
            ],
        }

    def generate_widget_info(self):
        project_name = self.request.session.get("project_name")

        try:
            users = self.keystone.user_list()
        except Exception as e:
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
            return {"error": "Unable to list users"}
        try:
            projects = self.keystone.project_list()
        except Exception as e:
            log.exception("{}{}".format(_("Exception:").encode("UTF-8"), e))
            return {"error": "Unable to list projects"}

        return [
            {
                "type": "default",
                "name": "keystone",
                "title": str(_("Keystone")),
                "subtitle": str(_("Identity")),
                "color": "green",
                "icon": "fas fa-key",
                "url": reverse("projects", kwargs={"project": project_name}),
                "properties": [
                    {"name": str(_("projects")), "description": "", "value": len(projects)},
                    {"name": str(_("users")), "description": "", "value": len(users)},
                ],
                "buttons": [
                    {"name": str(_("Projects")), "url": reverse("projects", kwargs={"project": project_name})},
                    {"name": str(_("Users")), "url": reverse("admin_list_users", kwargs={"project": project_name})},
                ],
            }
        ]


class JsonInfoView(SuperUserMixin, ProjectCheckMixin, WithKeystoneMixin, View):
    def get(self, request, *args, **kwargs):
        ksinfo = KeystoneJsonInfo(keystone=self.keystone, request=request)

        return ksinfo.render(request)



# -*- coding:utf-8 -*-

from uuid import uuid4
from unittest import TestCase
from unittest.mock import patch, MagicMock
from keystoneclient.v3.projects import Project
from keystoneclient import exceptions

from django.conf import settings
from django.contrib.auth.models import User, Group

from identity.keystone import Keystone
from identity.tests.fakes import UserFactory, GroupFactory, FakeResource

from vault.tests.fakes import fake_request
from vault.models import GroupProjects


class TestKeystoneV3(TestCase):
    """ Test keystone version 3 """

    @classmethod
    def tearDownClass(cls):
        for item in [User, Group, GroupProjects]:
            item.objects.all().delete()

    def setUp(self):
        self.request = fake_request()

        self.project = Project('123', {
            u'id': str(uuid4()),
            u'name': 'project_test',
            u'description': 'project description',
            u'domain_id': 'default',
            u'enabled': True
        })

        self.mock_project_create = patch(
            'identity.keystone.Keystone.project_create').start()
        self.mock_project_create.return_value = self.project

        self.mock_project_update = patch(
            'identity.keystone.Keystone.project_update').start()
        self.mock_project_update.return_value = self.project

        fake_project = FakeResource(n='abcdefg', name='fake_project')
        fake_project.description = 'desc do fake'

        self.mock_project_get = patch(
            'identity.keystone.Keystone.project_get').start()
        self.mock_project_get.return_value = fake_project

        self.mock_keystone_conn = patch(
            'identity.keystone.Keystone._create_keystone_connection').start()

        self.group = GroupFactory(id=1)

    def tearDown(self):
        self.group.delete()
        patch.stopall()

    def test_user_list_valid_project_id(self):
        """
        Testa metodo de listagem de usuario para o caso de um project_id valido
        que possui usuarios
        """
        expected = ['uma', 'lista', 'qualquer']

        self.mock_keystone_conn.return_value.users.list.return_value = expected
        keystone = Keystone(self.request)

        computed = keystone.user_list(project_id='123454321')

        self.assertEqual(computed, expected)

    def test_user_list_valid_project_id_with_no_user(self):
        """
        Testa metodo de listagem de usuario para o caso de um project_id valido
        que NAO possui usuarios
        """
        expected = []

        self.mock_keystone_conn.return_value.users.list.return_value = expected
        keystone = Keystone(self.request)

        computed = keystone.user_list(project_id='123454321')

        self.assertEqual(computed, expected)

    def test_user_list_invalid_project(self):
        """
        Testa metodo de listagem de usuario para o caso de um project_id
        invalido
        """
        self.mock_keystone_conn.return_value.users.list.side_effect = exceptions.NotFound
        keystone = Keystone(self.request)

        self.assertRaises(exceptions.NotFound, keystone.user_list)

    @patch('identity.keystone.Keystone.add_user_role')
    def test_keystone_create_user_with_no_role(self, mock_add_user_role):
        """ Cadastra usuario sem setar role para nenhum project """

        keystone = Keystone(self.request)
        keystone.user_create('name', email='email@email.com',
                             password='password', project_id='project_id',
                             enabled=True)

        keystone.conn.users.create.assert_called_with('name', domain=None,
            email='email@email.com', enabled=True, password='password', project='project_id')
        self.assertFalse(mock_add_user_role.called)

    @patch('identity.keystone.Keystone.add_user_role')
    @patch('identity.keystone.Keystone.project_get')
    @patch('identity.keystone.Keystone.role_get')
    def test_keystone_create_user_and_set_role(self, mock_role_get, mock_project_get, mock_add_user_role):
        """ Cadastra usuario e seta role para project """

        mock_user_create = self.mock_keystone_conn.return_value.users.create

        fake_user = FakeResource('user_id', name='user_name')
        fake_role = FakeResource('role_id', name='role_name')
        fake_project = FakeResource('project_id', name='project_name')

        mock_user_create.return_value = fake_user
        mock_role_get.return_value = fake_role
        mock_project_get.return_value = fake_project

        keystone = Keystone(self.request, 'project_name')
        keystone.user_create('name', email='email@email.com',
                             password='password', project_id='project_id',
                             enabled=True, role_id='role_id')

        keystone.conn.users.create.assert_called_with('name', domain=None,
            email='email@email.com', enabled=True, password='password', project='project_id')
        mock_add_user_role.assert_called_with(fake_user, fake_project, fake_role)

    @patch('identity.keystone.settings.KEYSTONE_ROLE', 'e03a556b664d4a41aaf2c5b4518f33ae')
    @patch('vault.utils.encrypt_password')
    @patch('identity.keystone.Keystone.user_create')
    @patch('identity.keystone.Keystone.create_password')
    @patch('identity.keystone.Keystone.vault_group_project_create')
    def test_vault_create_project(self, mock_gp_create,
                                        mock_key_pass,
                                        mock_key_user,
                                        mock_encrypt_password):

        mock_key_user.return_value = FakeResource(n=self.project.id, name='u_{}'.format(self.project.name))
        mock_key_pass.return_value = 'password'

        mock_encrypt_password.return_value = b'123456'

        keystone = Keystone(self.request, project_name='project_name')

        expected = {
            'status': True,
            'project': self.mock_project_create.return_value,
            'user': mock_key_user.return_value,
            'password': mock_key_pass.return_value,
        }

        computed = keystone.vault_project_create(self.project.name, self.group.id,
                                                 description=self.project.description)

        # Project creation
        self.mock_project_create.assert_called_with(self.project.name,
                                                    description=self.project.description,
                                                    enabled=True)

        # User creation
        mock_key_user.assert_called_with(name='u_vault_{}'.format(self.project.name),
                                         email='',
                                         password='password',
                                         enabled=True,
                                         domain='default',
                                         project_id=self.project.id,
                                         role_id='e03a556b664d4a41aaf2c5b4518f33ae')

        mock_gp_create.assert_called_with(group_id=self.group.id,
                                          project_id=self.project.id,
                                          owner=1)

        self.assertEqual(computed, expected)

    @patch('identity.keystone.Keystone.user_create')
    def test_vault_create_project_forbidden_on_project_create(self, mock_key_user):

        self.mock_project_create.side_effect = exceptions.Forbidden
        mock_key_user.return_value = FakeResource(n=self.project.id, name='u_{}'.format(self.project.name))

        keystone = Keystone(self.request, project_name='project_name')

        expected = {'status': False, 'reason': 'Superuser required.'}
        computed = keystone.vault_project_create(self.project.name, 1, description=self.project.description)

        self.assertEqual(computed, expected)

        # Criacao de user nao pode ocorrer quando hover falha na criacao do project
        self.assertFalse(mock_key_user.called)

    @patch('identity.keystone.Keystone.project_create')
    @patch('identity.keystone.Keystone.project_delete')
    @patch('identity.keystone.Keystone.user_create')
    def test_vault_create_project_forbidden_on_user_create(self, mock_key_user,
                                                                 mock_project_delete,
                                                                 mock_project_create):

        mock_project_create.return_value = Project('123',
            {u'id': self.project.id, u'name': self.project.name})
        mock_key_user.side_effect = exceptions.Forbidden

        keystone = Keystone(self.request, project_name='project_name')

        expected = {'status': False, 'reason': 'Admin User required'}
        computed = keystone.vault_project_create(self.project.name, 1, description=self.project.description)

        self.assertEqual(computed, expected)

        # Se falhou o cadastro de usuario, o project devera ser deletado
        mock_project_delete.assert_called_with(self.project.id)

    @patch('identity.keystone.settings.KEYSTONE_ROLE', 'e03a556b664d4a41aaf2c5b4518f33ae')
    @patch('vault.utils.encrypt_password')
    @patch('identity.keystone.Keystone.project_delete')
    @patch('identity.keystone.Keystone.user_create')
    @patch('identity.keystone.Keystone.user_delete')
    @patch('identity.keystone.GroupProjects.save')
    def test_vault_create_project_fail_to_save_group_project_on_db(self, mock_gp_save,
                                                                         mock_user_delete,
                                                                         mock_user_create,
                                                                         mock_project_delete,
                                                                         mock_encrypt_password):

        fake_user = FakeResource(n=self.project.id, name='u_{}'.format(self.project.name))
        mock_user_create.return_value = fake_user

        # Excecao ao salvar no db
        mock_gp_save.side_effect = Exception

        mock_encrypt_password.return_value = b'123456'

        keystone = Keystone(self.request, project_name='project_name')

        expected = {'status': False, 'reason': 'Unable to assign project to group'}
        computed = keystone.vault_project_create(self.project.name, 2, description=self.project.description)

        self.assertEqual(computed, expected)

        mock_project_delete.assert_called_with(self.project.id)

        mock_user_delete.assert_called_with(fake_user.id)

    @patch('identity.keystone.Keystone.vault_set_project_owner')
    def test_vault_update_project_keystone(self, mock_prj_owner):
        mock_prj_owner.return_value = {'status': True}
        fake_project = self.mock_project_get.return_value

        keystone = Keystone(self.request, project_name='project_name')
        computed = keystone.vault_project_update(self.project.id,
                                                 self.project.name,
                                                 self.group.id,
                                                 description=self.project.description)
        expected = {'status': True,
                    'project': self.mock_project_update.return_value}

        self.assertEqual(computed, expected)


class TestKeystoneDeleteProject(TestCase):

    def setUp(self):
        self.request = fake_request()

        self.user_id = 'abcdefghiklmnopq'
        self.user_name = 'user_name'

        self.project = Project('321', {
            u'id': str(uuid4()),
            u'name': 'project_test_delete',
            u'description': 'project test delete description',
            u'enabled': True
        })

        self.mock_keystone_conn = patch('identity.keystone.Keystone._create_keystone_connection').start()

        self.mock_project_get_by_name = patch('identity.keystone.Keystone.project_get_by_name').start()
        self.mock_project_get_by_name.return_value = self.project

        self.mock_find_user_with_u_prefix = patch('identity.keystone.Keystone.find_user_with_u_prefix').start()
        self.mock_find_user_with_u_prefix.return_value = FakeResource(self.user_id, name=self.user_name)

    def tearDown(self):
        patch.stopall()

    @patch('identity.keystone.Keystone.project_delete')
    @patch('identity.keystone.Keystone._project_delete_swift')
    @patch('identity.keystone.Keystone.get_object_store_endpoints')
    def test_vault_delete_project(self, mock_endpoints,
                                        mock_swift_delete,
                                        mock_keystone_delete):
        mock_endpoints.return_value = {'adminURL': 'https://fake.api.globoi.com/v1/AUTH_XPTO'}

        class FakeResponse:
            status_code = 204
        mock_swift_delete.return_value = FakeResponse()

        keystone = Keystone(self.request)
        _ = keystone.vault_project_delete(self.project.name)

        # Swift project delete
        mock_swift_delete.assert_called_with(self.project.id)

        # Find project's user
        self.mock_find_user_with_u_prefix.assert_called_with(self.project.id, 'u_vault')

        # Keystone project delete
        mock_keystone_delete.assert_called_with(self.project.id)


class TestKeystonePermissionToConnect(TestCase):
    """
    This tests are separated because it can't mock mock_keystone_is_allowed
    method
    """

    def setUp(self):
        self.request = fake_request()
        self.mock_filter = patch('identity.keystone.GroupProjects.objects.filter').start()
        patch('identity.keystone.Keystone._create_keystone_connection').start()

    def tearDown(self):
        patch.stopall()

    def test_superuser_creates_keystone_conn(self):
        self.mock_filter.return_value = None
        self.conn = Keystone(self.request, project_name='project_name')
        self.assertTrue(isinstance(self.conn, Keystone))

    def test_regular_user_creates_keystone_conn_on_a_allowed_project(self):

        # Se este mock retorna uma lista nao vazia, significa que o time do
        # usuario possui permissao no project
        self.mock_filter.return_value = [1]

        self.request.user.is_superuser = False
        self.conn = Keystone(self.request, project_name='project_name')

        self.assertTrue(isinstance(self.conn, Keystone))

    def test_regular_user_creates_keystone_conn_on_a_NOT_allowed_project(self):

        # Se este mock retorna uma lista  vazia, significa que o time do
        # usuario NAO possui permissao no project
        self.mock_filter.return_value = []
        self.request.user.is_superuser = False

        keystone = Keystone(self.request, project_name='abcdefg')
        self.assertEqual(keystone.conn, None)



# -*- coding: utf-8 -*-

from django.db import models


class BackupContainer(models.Model):
    id = models.AutoField(primary_key=True)
    container = models.CharField(max_length=255)
    project_id = models.CharField(max_length=255)
    project_name = models.CharField(max_length=255)
    created_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = 'swift_backup_container'
        unique_together = ('container', 'project_id',)

    def __unicode__(self):
        return "Container: {}, Project: {}".format(self.container,
                                                   self.project_name)



# -*- coding: utf-8 -*-

import syslog
from .models import Audit


class ActionNotFound(Exception):
    pass


class ActionLogger:
    """ A wrapper to log actions """

    def __init__(self):
        self._actions = {
            "create": "Criou",
            "update": "Atualizou",
            "delete": "Removeu",
            "upload": "Realizou Upload",
            "download": "Realizou Download",
            "enable": "Habilitou",
            "disable": "Desabilitou",
            "restore": "Restaurou",
            "update_trash": "Atualizou a lixeira do container",
            "remove_cache": "Removeu do cache",
            "update header": "Atualizou Header",
            "set_private": "Setou container como privado",
            "set_public": "Setou container como publico",
        }

    def log(self, user, action, item):
        if action not in self._actions.keys():
            raise ActionNotFound("Invalid action: '{}'".format(action))

        audit = Audit(user=user,
                      action=self._actions[action],
                      item=str(item))
        audit.save()

        msg = self._make_log_message(user, action, item)
        syslog.syslog(syslog.LOG_INFO, msg)

    def _make_log_message(self, user, action, item):
        return 'Usuario {} {} {}'.format(user,
                                         self._actions[action],
                                         str(item))



# -*- coding: utf-8 -*-

from unittest.mock import patch, Mock
from unittest import TestCase

from storage.tests import fakes
from storage.views import backup

from vault.tests.fakes import fake_request


class TestSwiftBackup(TestCase):

    def setUp(self):
        self.request = fake_request(user=False)

        # silence log
        patch('storage.views.backup.log',
              Mock(return_value=None)).start()

    def tearDown(self):
        patch.stopall()

    def test_authenticated_config_backup_url(self):
        response = backup.config_backup_container(self.request, 'blah')

        self.assertEqual(response.status_code, 302)



from django.apps import apps
from django.contrib import admin
from django.conf import settings
from django.conf.urls import include, re_path
from django.utils.translation import gettext_lazy as _

from identity.views import CreateProjectView, ChangeProjectView

from vault import views
from vault.forms import VaultLoginForm

admin.site.index_title = _("Admin Dashboard")


urlpatterns = [
    # overwrite admin:login
    re_path(r"^admin/login/$", views.VaultLogin.as_view(authentication_form=VaultLoginForm), name="vault_login"),

    # overwrite admin:logout (need to be before admin urls)
    re_path(r"^admin/logout/$", views.VaultLogout.as_view(), name="vault_logout"),

    # OAuthCallback
    re_path(r"^accounts/callback/(?P<provider>[\w\-]+)/$", views.OAuthVaultCallback.as_view(), name="allaccess-callback"),

    # OAuthLogin
    re_path(r"^accounts/login/(?P<provider>[\w\-]+)/$", views.OAuthVaultRedirect.as_view(), name="allaccess-login"),

    # OAuth
    re_path(r"^accounts/", include("allaccess.urls")),

    # Admin
    re_path(r"^admin/", admin.site.urls),

    # Team CRUD
    re_path(r"^team/add/user/?$", views.AddUserTeamView.as_view(), name="team_add_user"),
    re_path(r"^team/delete/user/?$", views.DeleteUserTeamView.as_view(), name="team_delete_user"),
    re_path(r"^team/list/users/?$", views.ListUserTeamView.as_view(), name="team_list_users"),
    re_path(r"^p/(?P<project>.+?)/team/update/users/?$", views.UpdateTeamsUsersView.as_view(), name="update_teams_users"),

    # Team List
    re_path(r"^p/(?P<project>.+?)/team-users/?$", views.ListUsersTeamsView.as_view(), name="team_users"),

    # Project
    re_path(r"^project/add/?$", CreateProjectView.as_view(), name="add_project"),
    re_path(r"^project/change/?$", ChangeProjectView.as_view(), name="change_project"),
    re_path(r"^project/(?P<project_id>[\w\-]+)/set/?$", views.SetProjectView.as_view(), name="set_project"),

    # Team Manage
    re_path(r"^p/(?P<project>.+?)/team/manage/?$", views.team_manager_view, name="team_manage"),
    re_path(r"^p/(?P<project>.+?)/team/manage/outsideusers/?$", views.list_users_outside_a_group, name="outside_users"),

    # Apps Info
    re_path(r"^apps/info", views.apps_info, name="apps_info"),
]


# Swift Cloud
if settings.SWIFT_CLOUD_ENABLED:
    urlpatterns.append(re_path(r"^swift-cloud/report/?$", views.swift_cloud_report, name="swift_cloud_report"))
    urlpatterns.append(re_path(r"^swift-cloud/status/?$", views.swift_cloud_status, name="swift_cloud_status"))
    urlpatterns.append(re_path(r"^swift-cloud/migrate/?$", views.swift_cloud_migrate, name="swift_cloud_migrate"))


for app in apps.app_configs:
    if "vault_app" in dir(apps.app_configs[app]):
        urlpatterns.append(re_path(rf"^p/(?P<project>.+?)/{app}/", include(f"{app}.urls")))


urlpatterns.append(re_path(r"^p/(?P<project>.+?)/", views.DashboardView.as_view(), name="dashboard"))
urlpatterns.append(re_path(r"^", views.main_page, name="main"))



import json
import logging
import requests
from django.conf import settings
from django.shortcuts import render
from django.http import JsonResponse
from django.contrib.auth.decorators import login_required
from identity.keystone import Keystone

log = logging.getLogger(__name__)


@login_required
def swift_cloud_report(request):
    keystone = Keystone(request)
    projects = []
    environ = settings.ENVIRON

    if not environ and "localhost" in request.get_host():
        environ = "local"

    try:
        for project in keystone.project_list():
            projects.append({
                "id": project.id,
                "name": project.name,
                "description": project.description,
                "environment": environ,
                "status": "",
            })
    except Exception as e:
        log.exception(f"Keystone error: {e}")

    context = {"projects": json.dumps(projects)}

    return render(request, "vault/swift_cloud/report.html", context)


@login_required
def swift_cloud_status(request):
    project_id = request.GET.get('project_id')

    if not project_id:
        return JsonResponse({"error": "Missing project_id parameter"}, status=400)

    url = f"{settings.SWIFT_CLOUD_TOOLS_URL}/transfer/{project_id}"
    headers = {"X-Auth-Token": settings.SWIFT_CLOUD_TOOLS_API_KEY}
    content = {"status": None}

    try:
        res = requests.get(url, headers=headers)
        data = res.json()

        if res.status_code == 404:
            content["status"] = "Not initialized"
        else:
            content["status"] = "Waiting"

        if data.get("initial_date") and not data.get("final_date"):
            content["status"] = "Migrating"

        if data.get("final_date"):
            content["status"] = "Done"
    except Exception as err:
        log.error(err)

    return JsonResponse(content, status=200)


@login_required
def swift_cloud_migrate(request):
    if request.method != 'POST':
        return JsonResponse({"error": "Method not allowed"}, status=405)

    url = f"{settings.SWIFT_CLOUD_TOOLS_URL}/transfer/"
    headers = {"X-Auth-Token": settings.SWIFT_CLOUD_TOOLS_API_KEY}
    params = json.loads(request.body)
    content = {"message": "Migration job created"}
    status = 201

    try:
        res = requests.post(url, json=params, headers=headers)
        status = res.status_code
        if status != 201:
            content = {"error": res.text}
    except Exception as err:
        log.error(err)
        content = {"error": err}

    return JsonResponse(content, status=status)
